@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: rclconfig.cpp,v 1.43 2007/02/07 17:17:11 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: rclconfig.cpp,v 1.55 2007/11/16 14:28:52 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -39,6 +39,7 @@ static char rcsid[] = "@(#$Id: rclconfig.cpp,v 1.43 2007/02/07 17:17:11 dockes E
 #include "conftree.h"
 #include "debuglog.h"
 #include "smallut.h"
+#include "textsplit.h"
 
 #ifndef NO_NAMESPACES
 using namespace std;
@@ -102,39 +103,99 @@ RclConfig::RclConfig(const string *argcnf)
 	    return;
     }
 
-    list<string> cdirs;
-    cdirs.push_back(m_confdir);
-    cdirs.push_back(path_cat(m_datadir, "examples"));
+    m_cdirs.push_back(m_confdir);
+    m_cdirs.push_back(path_cat(m_datadir, "examples"));
+    string cnferrloc = m_confdir + " or " + path_cat(m_datadir, "examples");
 
-    m_conf = new ConfStack<ConfTree>("recoll.conf", cdirs, true);
-    if (m_conf == 0 || !m_conf->ok()) {
-	m_reason = string("No/bad main configuration file in: ") + m_confdir;
+    if (!updateMainConfig())
 	return;
-    }
 
-    mimemap = new ConfStack<ConfTree>("mimemap", cdirs, true);
+    mimemap = new ConfStack<ConfTree>("mimemap", m_cdirs, true);
     if (mimemap == 0 || !mimemap->ok()) {
-	m_reason = string("No or bad mimemap file in: ") + m_confdir;
+	m_reason = string("No or bad mimemap file in: ") + cnferrloc;
 	return;
     }
 
-    mimeconf = new ConfStack<ConfTree>("mimeconf", cdirs, true);
+    mimeconf = new ConfStack<ConfTree>("mimeconf", m_cdirs, true);
     if (mimeconf == 0 || !mimeconf->ok()) {
-	m_reason = string("No/bad mimeconf in: ") + m_confdir;
+	m_reason = string("No/bad mimeconf in: ") + cnferrloc;
 	return;
     }
-    mimeview = new ConfStack<ConfTree>("mimeview", cdirs, true);
+    mimeview = new ConfStack<ConfTree>("mimeview", m_cdirs, true);
     if (mimeconf == 0 || !mimeconf->ok()) {
-	m_reason = string("No/bad mimeview in: ") + m_confdir;
+	m_reason = string("No/bad mimeview in: ") + cnferrloc;
 	return;
     }
 
-    setKeyDir("");
-
     m_ok = true;
+    setKeyDir("");
     return;
 }
 
+bool RclConfig::updateMainConfig()
+{
+    m_conf = new ConfStack<ConfTree>("recoll.conf", m_cdirs, true);
+    if (m_conf == 0 || !m_conf->ok()) {
+	string where;
+	stringsToString(m_cdirs, where);
+	m_reason = string("No/bad main configuration file in: ") + where;
+	m_ok = false;
+	return false;
+    }
+    setKeyDir("");
+    bool nocjk = false;
+    if (getConfParam("nocjk", &nocjk) && nocjk == true) {
+	TextSplit::cjkProcessing(false);
+    } else {
+	int ngramlen;
+	if (getConfParam("cjkngramlen", &ngramlen)) {
+	    TextSplit::cjkProcessing(true, (unsigned int)ngramlen);
+	} else {
+	    TextSplit::cjkProcessing(true);
+	}
+    }
+    return true;
+}
+
+ConfNull *RclConfig::cloneMainConfig()
+{
+    ConfNull *conf = new ConfStack<ConfTree>("recoll.conf", m_cdirs, false);
+    if (conf == 0 || !conf->ok()) {
+	m_reason = string("Can't read config");
+	return 0;
+    }
+    return conf;
+}
+
+// Remember what directory we're under (for further conf->get()s), and 
+// prefetch a few common values.
+void RclConfig::setKeyDir(const string &dir) 
+{
+    m_keydir = dir;
+    if (m_conf == 0)
+	return;
+
+    if (!m_conf->get("defaultcharset", defcharset, m_keydir))
+	defcharset.erase();
+
+    getConfParam("guesscharset", &guesscharset);
+
+    string rmtstr;
+    if (m_conf->get("indexedmimetypes", rmtstr, m_keydir)) {
+	stringtolower(rmtstr);
+	if (rmtstr != m_rmtstr) {
+	    LOGDEB2(("RclConfig::setKeyDir: rmtstr [%s]\n", rmtstr.c_str()));
+	    m_rmtstr = rmtstr;
+	    list<string> l;
+	    // Yea, no good to go string->list->set. Lazy me.
+	    stringToStrings(rmtstr, l);
+	    for (list<string>::iterator it = l.begin(); it !=l.end(); it++) {
+		m_restrictMTypes.insert(*it);
+	    }
+	}
+    }
+}
+
 bool RclConfig::getConfParam(const std::string &name, int *ivp)
 {
     string value;
@@ -304,9 +365,10 @@ bool RclConfig::inStopSuffixes(const string& fni)
 	}
     }
 
-    string fn(fni, 
-	      MAX(0, fni.length() - m_maxsufflen), 
-	      MIN(fni.length(), m_maxsufflen));
+    // Only need a tail as long as the longest suffix.
+    int pos = MAX(0, int(fni.length() - m_maxsufflen));
+    string fn(fni, pos);
+
     stringtolower(fn);
     SuffixStore::const_iterator it = STOPSUFFIXES->find(fn);
     if (it != STOPSUFFIXES->end()) {
@@ -349,6 +411,17 @@ bool RclConfig::getMimeCategories(list<string>& cats)
     return true;
 }
 
+bool RclConfig::isMimeCategory(string& cat) 
+{
+    list<string>cats;
+    getMimeCategories(cats);
+    for (list<string>::iterator it = cats.begin(); it != cats.end(); it++) {
+	if (!stringicmp(*it,cat))
+	    return true;
+    }
+    return false;
+}
+
 /** Get list of mime types for category from mimeconf */
 bool RclConfig::getMimeCatTypes(const string& cat, list<string>& tps)
 {
@@ -363,15 +436,30 @@ bool RclConfig::getMimeCatTypes(const string& cat, list<string>& tps)
     return true;
 }
 
-string RclConfig::getMimeHandlerDef(const std::string &mtype)
+string RclConfig::getMimeHandlerDef(const std::string &mtype, bool filtertypes)
 {
     string hs;
+    if (filtertypes && !m_restrictMTypes.empty()) {
+	string mt = mtype;
+	stringtolower(mt);
+	if (m_restrictMTypes.find(mt) == m_restrictMTypes.end())
+	    return hs;
+    }
     if (!mimeconf->get(mtype, hs, "index")) {
 	LOGDEB1(("getMimeHandler: no handler for '%s'\n", mtype.c_str()));
     }
     return hs;
 }
 
+bool RclConfig::getFieldPrefix(const string& fld, string &pfx)
+{
+    if (!mimeconf->get(fld, pfx, "prefixes")) {
+      LOGDEB(("getFieldPrefix: no prefix defined for '%s'\n", fld.c_str()));
+      return false;
+    }
+    return true;
+}
+
 string RclConfig::getMimeViewerDef(const string &mtype)
 {
     string hs;
@@ -461,6 +549,11 @@ string RclConfig::getDbDir()
     return path_canon(dbdir);
 }
 
+string RclConfig::getStopfile()
+{
+    return path_cat(getConfDir(), "stoplist.txt");
+}
+
 list<string> RclConfig::getSkippedNames()
 {
     list<string> skpl;
@@ -660,6 +753,8 @@ void RclConfig::initFrom(const RclConfig& r)
     m_maxsufflen = r.m_maxsufflen;
     defcharset = r.defcharset;
     guesscharset = r.guesscharset;
+    m_rmtstr = r.m_rmtstr;
+    m_restrictMTypes = r.m_restrictMTypes;
 }
 
 #else // -> Test
