@@ -16,17 +16,19 @@
  */
 #ifndef _RCLCONFIG_H_INCLUDED_
 #define _RCLCONFIG_H_INCLUDED_
-/* @(#$Id: rclconfig.h,v 1.31 2007/02/02 10:12:58 dockes Exp $  (C) 2004 J.F.Dockes */
+/* @(#$Id: rclconfig.h,v 1.39 2007/11/16 14:28:52 dockes Exp $  (C) 2004 J.F.Dockes */
 
 #include <list>
 #include <string>
 #include <vector>
+#include <set>
 #include <utility>
 #ifndef NO_NAMESPACES
 using std::list;
 using std::string;
 using std::vector;
 using std::pair;
+using std::set;
 #endif
 
 #include "conftree.h"
@@ -35,22 +37,32 @@ using std::pair;
 class RclConfig {
  public:
 
+    // Constructor: we normally look for a configuration file, except
+    // if this was specified on the command line and passed through
+    // argcnf
     RclConfig(const string *argcnf = 0);
+
+    // Main programs must implement this, it avoids having to carry
+    // the configuration parameter everywhere. Places where several
+    // RclConfig instances might be needed will take care of
+    // themselves.
+    static RclConfig* getMainConfig();
+
+    // Return a writable clone of the main config. This belongs to the
+    // caller (must delete it when done)
+    ConfNull *cloneMainConfig();
+
+    /** (re)Read recoll.conf */
+    bool updateMainConfig();
+
     bool ok() {return m_ok;}
     const string &getReason() {return m_reason;}
-    /** Return the directory where this config is stored */
+
+    /** Return the directory where this configuration is stored */
     string getConfDir() {return m_confdir;}
 
     /** Set current directory reference, and fetch automatic parameters. */
-    void setKeyDir(const string &dir) 
-    {
-	m_keydir = dir;
-	if (!m_conf->get("defaultcharset", defcharset, m_keydir))
-	    defcharset.erase();
-	string str;
-	m_conf->get("guesscharset", str, m_keydir);
-	guesscharset = stringToBool(str);
-    }
+    void setKeyDir(const string &dir);
     string getKeyDir() const {return m_keydir;}
 
     /** Get generic configuration parameter according to current keydir */
@@ -78,8 +90,10 @@ class RclConfig {
 
     /** Get database directory */
     string getDbDir();
+    /** Get stoplist file name */
+    string getStopfile();
 
-    /** Get list of skipped names for current keydir */
+    /** Get list of skipped file names for current keydir */
     list<string> getSkippedNames();
 
     /** Get list of skipped paths patterns. Doesn't depend on the keydir */
@@ -106,30 +120,34 @@ class RclConfig {
      */
     bool getUncompressor(const string &mtpe, list<string>& cmd);
 
-    /** Use mimemap to compute mimetype */
+    /** mimemap: compute mimetype */
     string getMimeTypeFromSuffix(const string &suffix);
-
-    /** Get appropriate suffix for mime type. This is inefficient */
+    /** mimemap: get a list of all indexable mime types defined */
+    list<string> getAllMimeTypes();
+    /** mimemap: Get appropriate suffix for mime type. This is inefficient */
     string getSuffixFromMimeType(const string &mt);
 
-    /** Get input filter from mimeconf for mimetype */
-    string getMimeHandlerDef(const string &mimetype);
+    /** mimeconf: get input filter for mimetype */
+    string getMimeHandlerDef(const string &mimetype, bool filtertypes=false);
 
-    /** Get external viewer exec string from mimeconf for mimetype */
+    /** mimeconf: get icon name for mimetype */
+    string getMimeIconName(const string &mtype, string *path = 0);
+
+    /** mimeconf: get list of file categories */
+    bool getMimeCategories(list<string>&);
+    /** mimeconf: is parameter one of the categories ? */
+    bool isMimeCategory(string&);
+    /** mimeconf: get list of mime types for category */
+    bool getMimeCatTypes(const string& cat, list<string>&);
+
+    /** mimeconf: get field prefix from field name */
+    bool getFieldPrefix(const string& fldname, string &pfx);
+
+    /** mimeview: get/set external viewer exec string(s) for mimetype(s) */
     string getMimeViewerDef(const string &mimetype);
     bool getMimeViewerDefs(vector<pair<string, string> >&);
     bool setMimeViewerDef(const string& mimetype, const string& cmd);
 
-    /** Get icon name from mimeconf for mimetype */
-    string getMimeIconName(const string &mtype, string *path = 0);
-
-    /** Get list of file categories from mimeconf */
-    bool getMimeCategories(list<string>&);
-    /** Get list of mime types for category from mimeconf */
-    bool getMimeCatTypes(const string& cat, list<string>&);
-
-    /** Get a list of all indexable mime types defined in mimemap */
-    list<string> getAllMimeTypes();
 
     /** Find exec file for external filter. cmd is the command name from the
      * command string returned by getMimeHandlerDef */
@@ -156,9 +174,10 @@ class RclConfig {
  private:
     int m_ok;
     string m_reason;    // Explanation for bad state
-    string m_confdir; // Directory where the files are stored
-    string m_datadir; // Example: /usr/local/share/recoll
+    string m_confdir;   // User directory where the customized files are stored
+    string m_datadir;   // Example: /usr/local/share/recoll
     string m_keydir;    // Current directory used for parameter fetches.
+    list<string> m_cdirs; // directory stack for the confstacks
 
     ConfStack<ConfTree> *m_conf;   // Parsed configuration files
     ConfStack<ConfTree> *mimemap;  // The files don't change with keydir, 
@@ -171,10 +190,12 @@ class RclConfig {
     // Parameters auto-fetched on setkeydir
     string defcharset;   // These are stored locally to avoid 
     bool   guesscharset; // They are fetched initially or on setKeydir()
+    // Limiting set of mime types to be processed. Normally empty.
+    string        m_rmtstr;
+    set<string>   m_restrictMTypes; 
 
     /** Create initial user configuration */
     bool initUserConfig();
-
     /** Copy from other */
     void initFrom(const RclConfig& r);
     /** Init pointers to 0 */
