@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: rclinit.cpp,v 1.8 2006/11/08 15:34:20 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: rclinit.cpp,v 1.10 2007/05/23 08:28:35 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@ static char rcsid[] = "@(#$Id: rclinit.cpp,v 1.8 2006/11/08 15:34:20 dockes Exp
 #include <stdio.h>
 #include <signal.h>
 #include <locale.h>
+#include <pthread.h>
 
 #include "debuglog.h"
 #include "rclconfig.h"
@@ -36,6 +37,14 @@ RclConfig *recollinit(RclInitFlags flags,
     if (cleanup)
 	atexit(cleanup);
 
+    // We ignore SIGPIPE always. All pieces of code which can write to a pipe
+    // must check write() return values.
+    signal(SIGPIPE, SIG_IGN);
+    
+    // We would like to block SIGCHLD globally, but we can't because
+    // QT uses it. Have to block it inside execmd.cpp
+
+    // Install signal handler
     if (sigcleanup) {
 	for (unsigned int i = 0; i < sizeof(catchedSigs) / sizeof(int); i++)
 	    if (signal(catchedSigs[i], SIG_IGN) != SIG_IGN)
@@ -86,3 +95,15 @@ RclConfig *recollinit(RclInitFlags flags,
 
     return config;
 }
+
+// Signals are handled by the main thread. All others should call this routine
+// to block possible signals
+void recoll_threadinit()
+{
+    sigset_t sset;
+    sigemptyset(&sset);
+
+    for (unsigned int i = 0; i < sizeof(catchedSigs) / sizeof(int); i++)
+	sigaddset(&sset, catchedSigs[i]);
+    pthread_sigmask(SIG_BLOCK, &sset, 0);
+}
