@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: indexer.cpp,v 1.53 2007/02/08 17:05:12 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: indexer.cpp,v 1.63 2007/10/04 12:20:56 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -71,6 +71,11 @@ DbIndexer::~DbIndexer() {
     m_db.close();
 }
 
+list<string> DbIndexer::getStemmerNames()
+{
+    return Rcl::Db::getStemmerNames();
+}
+
 // Index each directory in the topdirs for a given db
 bool DbIndexer::indexDb(bool resetbefore, list<string> *topdirs)
 {
@@ -92,6 +97,15 @@ bool DbIndexer::indexDb(bool resetbefore, list<string> *topdirs)
 	// Set the current directory in config so that subsequent
 	// getConfParams() will get local values
 	m_config->setKeyDir(*it);
+
+	// Adjust the "follow symlinks" option
+	bool follow;
+	if (m_config->getConfParam("followLinks", &follow) && follow) {
+	    m_walker.setOpts(FsTreeWalker::FtwFollow);
+	} else {
+	    m_walker.setOpts(FsTreeWalker::FtwOptNone);
+	}	    
+
 	int abslen;
 	if (m_config->getConfParam("idxabsmlen", &abslen))
 	    m_db.setAbstractParams(abslen, -1, -1);
@@ -134,8 +148,8 @@ bool DbIndexer::indexDb(bool resetbefore, list<string> *topdirs)
     if (!m_missingExternal.empty()) {
 	string missing;
 	stringsToString(m_missingExternal, missing);
-	LOGERR(("DbIndexer::index missing helper program(s): %s\n", 
-		missing.c_str()));
+	LOGINFO(("DbIndexer::index missing helper program(s): %s\n", 
+		 missing.c_str()));
     }
     return true;
 }
@@ -181,10 +195,11 @@ bool DbIndexer::init(bool resetbefore, bool rdonly)
     }
     Rcl::Db::OpenMode mode = rdonly ? Rcl::Db::DbRO :
 	resetbefore ? Rcl::Db::DbTrunc : Rcl::Db::DbUpd;
-    if (!m_db.open(m_dbdir, mode)) {
+    if (!m_db.open(m_dbdir, m_config->getStopfile(), mode)) {
 	LOGERR(("DbIndexer: error opening database in %s\n", m_dbdir.c_str()));
 	return false;
     }
+
     return true;
 }
 
@@ -201,8 +216,15 @@ bool DbIndexer::createAspellDict()
 {
     LOGDEB2(("DbIndexer::createAspellDict()\n"));
 #ifdef RCL_USE_ASPELL
-    bool noaspell = false;
-    m_config->getConfParam("noaspell", &noaspell);
+    // For the benefit of the real-time indexer, we only initialize
+    // noaspell from the configuration once. It can then be set to
+    // true if dictionary generation fails, which avoids retrying
+    // it forever.
+    static int noaspell = -12345;
+    if (noaspell == -12345) {
+	noaspell = false;
+	m_config->getConfParam("noaspell", &noaspell);
+    }
     if (noaspell)
 	return true;
 
@@ -213,18 +235,21 @@ bool DbIndexer::createAspellDict()
     if (!aspell.init(reason)) {
 	LOGERR(("DbIndexer::createAspellDict: aspell init failed: %s\n", 
 		reason.c_str()));
+	noaspell = true;
 	return false;
     }
     LOGDEB(("DbIndexer::createAspellDict: creating dictionary\n"));
     if (!aspell.buildDict(m_db, reason)) {
 	LOGERR(("DbIndexer::createAspellDict: aspell buildDict failed: %s\n", 
 		reason.c_str()));
+	noaspell = true;
 	return false;
     }
     // The close would be done in our destructor, but we want status here
     if (!m_db.close()) {
 	LOGERR(("DbIndexer::indexfiles: error closing database in %s\n", 
 		m_dbdir.c_str()));
+	noaspell = true;
 	return false;
     }
 #endif
@@ -379,8 +404,16 @@ DbIndexer::processone(const std::string &fn, const struct stat *stp,
     // If this fails, the file name won't be indexed, no big deal
     // Note that we used to do the full path here, but I ended up believing
     // that it made more sense to use only the file name
-    string utf8fn;
-    transcode(path_getsimple(fn), utf8fn, charset, "UTF-8");
+    string utf8fn; int ercnt;
+    if (!transcode(path_getsimple(fn), utf8fn, charset, "UTF-8", &ercnt)) {
+	LOGERR(("processone: fn transcode failure from [%s] to UTF-8: %s\n",
+		charset.c_str(), path_getsimple(fn).c_str()));
+    } else if (ercnt) {
+	LOGDEB(("processone: fn transcode %d errors from [%s] to UTF-8: %s\n",
+		ercnt, charset.c_str(), path_getsimple(fn).c_str()));
+    }
+    LOGDEB2(("processone: fn transcoded from [%s] to [%s] (%s->%s)\n",
+	    path_getsimple(fn).c_str(), utf8fn.c_str(), charset.c_str(), "UTF-8"));
 
     FileInterner::Status fis = FileInterner::FIAgain;
     bool hadNullIpath = false;
@@ -477,21 +510,6 @@ bool ConfIndexer::index(bool resetbefore)
     for (dirit = tdl.begin(); dirit != tdl.end(); dirit++) {
 	string dbdir;
 	string doctopdir = *dirit;
-	{ // Check top dirs. Must not be symlinks
-	    struct stat st;
-	    if (lstat(doctopdir.c_str(), &st) < 0) {
-		LOGERR(("ConfIndexer::index: cant stat %s\n",
-			doctopdir.c_str()));
-		m_reason = "Stat error for: " + doctopdir;
-		return false;
-	    }
-	    if (S_ISLNK(st.st_mode)) {
-		LOGERR(("ConfIndexer::index: no symlinks allowed in topdirs: %s\n",
-			doctopdir.c_str()));
-		m_reason = doctopdir + " is a symbolic link";
-		return false;
-	    }
-	}
 	m_config->setKeyDir(doctopdir);
 	dbdir = m_config->getDbDir();
 	if (dbdir.empty()) {
