@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: recollindex.cpp,v 1.31 2007/02/02 10:09:10 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: recollindex.cpp,v 1.34 2007/07/10 09:23:28 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -74,8 +74,7 @@ static bool makeDbIndexer(RclConfig *config)
     }
     // Check if there is already an indexer for the right db
     if (dbindexer && dbindexer->getDbDir().compare(dbdir)) {
-	delete dbindexer;
-	dbindexer = 0;
+	deleteZ(dbindexer);
     }
 
     if (!dbindexer)
@@ -199,10 +198,8 @@ static bool createstemdb(RclConfig *config, const string &lang)
 
 static void cleanup()
 {
-    delete confindexer;
-    confindexer = 0;
-    delete dbindexer;
-    dbindexer = 0;
+    deleteZ(confindexer);
+    deleteZ(dbindexer);
 }
 
 static const char *thisprog;
@@ -219,6 +216,7 @@ static int     op_flags;
 #define OPT_e     0x200
 #define OPT_w     0x400
 #define OPT_x     0x800
+#define OPT_l     0x1000
 
 static const char usage [] =
 "\n"
@@ -237,6 +235,8 @@ static const char usage [] =
 "    Purge data for individual files. No stem database updates\n"
 "recollindex -i <filename [filename ...]>\n"
 "    Index individual files. No database purge or stem database updates\n"
+"recollindex -l\n"
+"    List available stemming languages\n"
 "recollindex -s <lang>\n"
 "    Build stem database for additional language <lang>\n"
 #ifdef RCL_USE_ASPELL
@@ -255,6 +255,12 @@ Usage(void)
     exit((op_flags & OPT_h)==0);
 }
 
+static RclConfig *config;
+RclConfig *RclConfig::getMainConfig() 
+{
+    return config;
+}
+
 int main(int argc, const char **argv)
 {
     string a_config;
@@ -279,6 +285,7 @@ int main(int argc, const char **argv)
 	    case 'e': op_flags |= OPT_e; break;
 	    case 'h': op_flags |= OPT_h; break;
 	    case 'i': op_flags |= OPT_i; break;
+	    case 'l': op_flags |= OPT_l; break;
 #ifdef RCL_MONITOR
 	    case 'm': op_flags |= OPT_m; break;
 #endif
@@ -306,8 +313,7 @@ int main(int argc, const char **argv)
     string reason;
     RclInitFlags flags = (op_flags & OPT_m) && !(op_flags&OPT_D) ? 
 	RCLINIT_DAEMON : RCLINIT_NONE;
-    RclConfig *config = 
-	recollinit(flags, cleanup, sigcleanup, reason, &a_config);
+    config = recollinit(flags, cleanup, sigcleanup, reason, &a_config);
     if (config == 0 || !config->ok()) {
 	cerr << "Configuration problem: " << reason << endl;
 	exit(1);
@@ -336,6 +342,15 @@ int main(int argc, const char **argv)
 	else 
 	    exit(!purgefiles(config, filenames));
 
+    } else if (op_flags & OPT_l) {
+	if (argc != 0) 
+	    Usage();
+	list<string> stemmers = DbIndexer::getStemmerNames();
+	for (list<string>::const_iterator it = stemmers.begin(); 
+	     it != stemmers.end(); it++) {
+	    cout << *it << endl;
+	}
+	exit(0);
     } else if (op_flags & OPT_s) {
 	if (argc != 1) 
 	    Usage();
@@ -360,15 +375,15 @@ int main(int argc, const char **argv)
 
 	confindexer = new ConfIndexer(config, &updater);
 	confindexer->index(rezero);
-	delete confindexer;
+	deleteZ(confindexer);
 	int opts = RCLMON_NONE;
 	if (op_flags & OPT_D)
 	    opts |= RCLMON_NOFORK;
 	if (op_flags & OPT_x)
 	    opts |= RCLMON_NOX11;
-	if (startMonitor(config, opts))
-	    exit(0);
-	exit(1);
+	bool monret = startMonitor(config, opts);
+	MONDEB(("Monitor returned %d, exiting\n", monret));
+	exit(monret == false);
 #endif // MONITOR
 
 #ifdef RCL_USE_ASPELL
