@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: internfile.cpp,v 1.28 2007/02/19 18:05:25 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: internfile.cpp,v 1.37 2007/11/16 14:28:52 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@ static char rcsid[] = "@(#$Id: internfile.cpp,v 1.28 2007/02/19 18:05:25 dockes
 
 #include <string>
 #include <iostream>
+#include <map>
 #ifndef NO_NAMESPACES
 using namespace std;
 #endif /* NO_NAMESPACES */
@@ -62,26 +63,12 @@ static bool uncompressfile(RclConfig *conf, const string& ifn,
     list<string>::const_iterator it = cmdv.begin();
     ++it;
     list<string> args;
+    map<char, string> subs;
+    subs['f'] = ifn;
+    subs['t'] = tdir;
     for (; it != cmdv.end(); it++) {
-	string s = *it;
 	string ns;
-	string::const_iterator it1;
-	for (it1 = s.begin(); it1 != s.end();it1++) {
-	    if (*it1 == '%') {
-		if (++it1 == s.end()) {
-		    ns += '%';
-		    break;
-		}
-		if (*it1 == '%')
-		    ns += '%';
-		if (*it1 == 'f')
-		    ns += ifn;
-		if (*it1 == 't')
-		    ns += tdir;
-	    } else {
-		ns += *it1;
-	    }
-	}
+	pcSubst(*it, ns, subs);
 	args.push_back(ns);
     }
 
@@ -158,7 +145,7 @@ FileInterner::FileInterner(const std::string &f, const struct stat *stp,
 
     // Look for appropriate handler (might still return empty)
     m_mimetype = l_mime;
-    Dijon::Filter *df = getMimeHandler(l_mime, m_cfg);
+    Dijon::Filter *df = getMimeHandler(l_mime, m_cfg, !m_forPreview);
 
     if (!df) {
 	// No handler for this type, for now :( if indexallfilenames
@@ -202,7 +189,8 @@ bool FileInterner::dataToTempFile(const string& dt, const string& mt,
 	m_tmpflgs[m_handlers.size()-1] = true;
 	m_tempfiles.push_back(temp);
     } else {
-	LOGERR(("FileInterner::dataToTempFile: cant create tempfile\n"));
+	LOGERR(("FileInterner::dataToTempFile: cant create tempfile: %s\n",
+		temp->getreason().c_str()));
 	return false;
     }
 
@@ -270,12 +258,12 @@ static const string keyab("abstract");
 static const string keyau("author");
 static const string keycs("charset");
 static const string keyct("content");
+static const string keyds("description");
 static const string keyfn("filename");
 static const string keykw("keywords");
 static const string keymd("modificationdate");
 static const string keymt("mimetype");
 static const string keyoc("origcharset");
-static const string keysm("sample");
 static const string keytt("title");
 
 bool FileInterner::dijontorcl(Rcl::Doc& doc)
@@ -283,15 +271,24 @@ bool FileInterner::dijontorcl(Rcl::Doc& doc)
     Dijon::Filter *df = m_handlers.back();
     const std::map<std::string, std::string>& docdata = df->get_meta_data();
 
-    getKeyValue(docdata, keyau, doc.author);
-    getKeyValue(docdata, keyoc, doc.origcharset);
-    getKeyValue(docdata, keyct, doc.text);    
-    getKeyValue(docdata, keytt, doc.title);
-    getKeyValue(docdata, keykw, doc.keywords);
-    getKeyValue(docdata, keymd, doc.dmtime);
-    if (!getKeyValue(docdata, keyab, doc.abstract))
-	getKeyValue(docdata, keysm, doc.abstract);
-    LOGDEB1(("FILENAME: %s\n", doc.utf8fn.c_str()));
+    for (map<string,string>::const_iterator it = docdata.begin(); 
+	 it != docdata.end(); it++) {
+	if (it->first == keyct) {
+	    doc.text = it->second;
+	} else if (it->first == keymd) {
+	    doc.dmtime = it->second;
+	} else if (it->first == keyoc) {
+	    doc.origcharset = it->second;
+	} else if (it->first == keymt || it->first == keycs) {
+	    // don't need these.
+	} else {
+	    doc.meta[it->first] = it->second;
+	}
+    }
+    if (doc.meta[keyab].empty() && !doc.meta[keyds].empty()) {
+	doc.meta[keyab] = doc.meta[keyds];
+	doc.meta.erase(keyds);
+    }
     return true;
 }
 
@@ -299,6 +296,8 @@ bool FileInterner::dijontorcl(Rcl::Doc& doc)
 // While we're at it, we also set the mimetype and filename, which are special 
 // properties: we want to get them from the topmost doc
 // with an ipath, not the last one which is usually text/plain
+// We also set the author and modification time from the last doc
+// which has them.
 void FileInterner::collectIpathAndMT(Rcl::Doc& doc, string& ipath) const
 {
     bool hasipath = false;
@@ -322,6 +321,8 @@ void FileInterner::collectIpathAndMT(Rcl::Doc& doc, string& ipath) const
 	} else {
 	    ipath += isep;
 	}
+	getKeyValue(docdata, keyau, doc.meta["author"]);
+	getKeyValue(docdata, keymd, doc.dmtime);
     }
 
     // Trim empty tail elements in ipath.
@@ -484,15 +485,19 @@ FileInterner::Status FileInterner::internfile(Rcl::Doc& doc, string& ipath)
     }
 
     if (m_handlers.empty()) {
-	LOGERR(("FileInterner::internfile: stack empty\n"));
+	LOGERR(("FileInterner::internfile: conversion ended with no doc\n"));
 	return FIError;
     }
 
-    // If indexing compute ipath and significant mimetype
-    // Note that ipath is returned through the parameter not doc.ipath
+    // If indexing compute ipath and significant mimetype Note that
+    // ipath is returned through the parameter not doc.ipath We also
+    // retrieve some metadata fields from the ancesters (like date or
+    // author). This is useful for email attachments. The values will
+    // be replaced by those found by dijontorcl if any, so the order
+    // of calls is important.
     if (!m_forPreview)
 	collectIpathAndMT(doc, ipath);
-
+    // Keep this AFTER collectIpathAndMT
     dijontorcl(doc);
 
     // Destack what can be
@@ -519,20 +524,26 @@ class DirWiper {
     }
 };
 
+// Extract subdoc out of multidoc into temporary file. 
+// We do the usual internfile stuff: create a temporary directory,
+// then create an interner and call internfile. 
+// We then write the data out of the resulting document into the output file.
 bool FileInterner::idocTempFile(TempFile& otemp, RclConfig *cnf, 
 				const string& fn,
 				const string& ipath,
 				const string& mtype)
 {
-    string tmpdir, reason;
-    if (!maketmpdir(tmpdir, reason))
-	return false;
-    DirWiper wiper(tmpdir);
     struct stat st;
     if (stat(fn.c_str(), &st) < 0) {
 	LOGERR(("FileInterner::idocTempFile: can't stat [%s]\n", fn.c_str()));
 	return false;
     }
+
+    string tmpdir, reason;
+    if (!maketmpdir(tmpdir, reason))
+	return false;
+    DirWiper wiper(tmpdir);
+
     FileInterner interner(fn, &st, cnf, tmpdir, &mtype);
     interner.setTargetMType(mtype);
     Rcl::Doc doc;
@@ -572,6 +583,8 @@ bool FileInterner::idocTempFile(TempFile& otemp, RclConfig *cnf,
 #include <stdlib.h>
 #include <iostream>
 #include <string>
+#include <sys/stat.h>
+
 using namespace std;
 
 #include "debuglog.h"
@@ -634,8 +647,12 @@ int main(int argc, char **argv)
 	fprintf(stderr, "%s\n", str.c_str());
 	exit(1);
     }
-
-    FileInterner interner(fn, config, "/tmp");
+    struct stat st;
+    if (stat(fn.c_str(), &st)) {
+	perror("stat");
+	exit(1);
+    }
+    FileInterner interner(fn, &st, config, "/tmp");
     Rcl::Doc doc;
     FileInterner::Status status = interner.internfile(doc, ipath);
     switch (status) {
@@ -660,11 +677,11 @@ int main(int argc, char **argv)
 	"]]]]\n-----------------------------------------------------\n" <<
 	"doc.origcharset [[[[" << doc.origcharset <<
 	"]]]]\n-----------------------------------------------------\n" <<
-	"doc.title [[[[" << doc.title <<
+	"doc.meta[title] [[[[" << doc.meta["title"] <<
 	"]]]]\n-----------------------------------------------------\n" <<
-	"doc.keywords [[[[" << doc.keywords <<
+	"doc.meta[keywords] [[[[" << doc.meta["keywords"] <<
 	"]]]]\n-----------------------------------------------------\n" <<
-	"doc.abstract [[[[" << doc.abstract <<
+	"doc.meta[abstract] [[[[" << doc.meta["abstract"] <<
 	"]]]]\n-----------------------------------------------------\n" <<
 	"doc.text [[[[" << doc.text << "]]]]\n";
 }
