@@ -17,7 +17,7 @@
 #ifndef _GUIUTILS_H_INCLUDED_
 #define _GUIUTILS_H_INCLUDED_
 /* 
- * @(#$Id: guiutils.h,v 1.20 2007/02/14 10:10:43 dockes Exp $  (C) 2005 Jean-Francois Dockes 
+ * @(#$Id: guiutils.h,v 1.25 2007/09/08 17:25:49 dockes Exp $  (C) 2005 Jean-Francois Dockes 
  *                         jean-francois.dockes@wanadoo.fr
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -61,9 +61,9 @@ extern bool printableUrl(const string &in, string &out);
 /** Holder for preferences (gets saved to user Qt prefs) */
 class PrefsPack {
  public:
-    bool showicons;
     bool autoSearchOnWS;
     int respagesize;
+    int maxhltextmbs;
     QString reslistfontfamily;
     int reslistfontsize;
     // Result list format string
@@ -76,6 +76,8 @@ class PrefsPack {
     int ssearchTyp;
     QString htmlBrowser;
     bool    useDesktopOpen; // typically xdg-open, instead of mimeview settings
+    bool    keepSort; // remember sort status between invocations
+    bool    sortActive; // Remembered sort state.
     bool queryBuildAbstract;
     bool queryReplaceAbstract;
     bool startWithAdvSearchOpen;
@@ -106,21 +108,35 @@ class PrefsPack {
     // Remembered term match mode
     int termMatchType;
 
+    // Program version that wrote this
+    int rclVersion;
+
     // Advanced search window clause list state
     vector<int> advSearchClauses;
 
     PrefsPack() :
-	showicons(true), 
 	respagesize(8), 
 	reslistfontsize(10),
 	ssearchTyp(0),
 	queryBuildAbstract(true),
 	queryReplaceAbstract(false),
 	startWithAdvSearchOpen(false),
-	    startWithSortToolOpen(false),
-	    termMatchType(0)
+	startWithSortToolOpen(false),
+	termMatchType(0),
+	rclVersion(1009)
 	    {
 	    }
+    static const char *getDfltResListFormat() {
+	return 	"<img src=\"%I\" align=\"left\">"
+	"%R %S %L &nbsp;&nbsp;<b>%T</b><br>"
+	"%M&nbsp;%D&nbsp;&nbsp;&nbsp;<i>%U</i><br>"
+	"%A %K";
+    }
+    static const char *getV18DfltResListFormat() {
+	return 	"%R %S %L &nbsp;&nbsp;<b>%T</b><br>"
+	"%M&nbsp;%D&nbsp;&nbsp;&nbsp;<i>%U</i><br>"
+	"%A %K";
+    }
 };
 
 /** Global preferences record */
@@ -129,4 +145,6 @@ extern PrefsPack prefs;
 /** Read write settings from disk file */
 extern void rwSettings(bool dowrite);
 
+extern QString g_stringAllStem, g_stringNoStem;
+
 #endif /* _GUIUTILS_H_INCLUDED_ */
