@@ -323,6 +323,61 @@ Click Cancel if you want to edit the configuration file before indexation starts
         <translation>Alt+C</translation>
     </message>
 </context>
+<context>
+    <name>QObject</name>
+    <message>
+        <source>Global parameters</source>
+        <translation>Parametres globaux</translation>
+    </message>
+    <message>
+        <source>Local parameters</source>
+        <translation>Parametres locaux</translation>
+    </message>
+    <message>
+        <source>&lt;b&gt;Customised subtrees</source>
+        <translation>&lt;b&gt;Repertoires avec parametres specifiques</translation>
+    </message>
+    <message>
+        <source>The list of subdirectories in the indexed hierarchy &lt;br&gt;where some parameters need to be redefined. Default: empty.</source>
+        <translation>La liste des sous-repertoires de la zone indexee&lt;br&gt;ou certains parametres sont redefinis. Defaut: vide.</translation>
+    </message>
+    <message>
+        <source>&lt;i&gt;The parameters that follow are set either at the top level, if nothing&lt;br&gt;or an empty line is selected in the listbox above, or for the selected subdirectory.&lt;br&gt;You can add or remove directories by clicking the +/- buttons.</source>
+        <translation>&lt;i&gt;Les parametres qui suivent sont definis soit globalement, si la selection dans la liste ci-dessus&lt;br&gt;est vide ou reduite a la ligne vide, soit pour le repertoire selectione.&lt;br&gt;Vous pouvez ajouter et enlever des repertoires en cliquant les boutons +/-.</translation>
+    </message>
+    <message>
+        <source>Skipped names</source>
+        <translation>Noms ignores</translation>
+    </message>
+    <message>
+        <source>These are patterns for file or directory  names which should not be indexed.</source>
+        <translation>Canevas definissant les fichiers ou repertoires qui ne doivent pas etre indexes.</translation>
+    </message>
+    <message>
+        <source>Default character set</source>
+        <translation>Jeu de caractere par defaut</translation>
+    </message>
+    <message>
+        <source>This is the character set used for reading files which do not identify the character set internally, for example pure text files.&lt;br&gt;The default value is empty, and the value from the NLS environnement is used.</source>
+        <translation>Jeu de caractere utilise pour lire les fichiers qui ne l&apos;identifient pas de maniere interne, par exemple les purs fichiers texte. &lt;br&gt;La valeur par defaut est vide, et le programme utilise l&apos;environnement.</translation>
+    </message>
+    <message>
+        <source>Follow symbolic links</source>
+        <translation>Suivre les liens symboliques</translation>
+    </message>
+    <message>
+        <source>Follow symbolic links while indexing. The default is no, to avoid duplicate indexing</source>
+        <translation>Indexer les fichiers et repertoires pointes par les liens symboliques. Pas fait par defaut pour eviter les indexations multiples</translation>
+    </message>
+    <message>
+        <source>Index all file names</source>
+        <translation>Indexer tous les noms de fichiers</translation>
+    </message>
+    <message>
+        <source>Index the names of files for which the contents cannot be identified or processed (no or unsupported mime type). Default true</source>
+        <translation>Indexer les noms des fichiers dont le contenu n&apos;est pas identifie ou traite (pas de type mime, ou type non supporte). Vrai par defaut</translation>
+    </message>
+</context>
 <context>
     <name>RclMain</name>
     <message>
@@ -463,6 +518,18 @@ Voulez vous ouvrir le dialogue de paramétrage ?</translation>
         <source>Cannot extract document or create temporary file</source>
         <translation>Impossible d&apos;extraire le document ou de créer le fichier temporaire</translation>
     </message>
+    <message>
+        <source>(no stemming)</source>
+        <translation>(pas d&apos;expansion)</translation>
+    </message>
+    <message>
+        <source>(all languages)</source>
+        <translation>(tous les langages)</translation>
+    </message>
+    <message>
+        <source>error retrieving stemming languages</source>
+        <translation>impossible de trouver la liste des langages d&apos;expansion</translation>
+    </message>
 </context>
 <context>
     <name>RclMainBase</name>
@@ -596,7 +663,7 @@ Voulez vous ouvrir le dialogue de paramétrage ?</translation>
     </message>
     <message>
         <source>&amp;Query configuration</source>
-        <translation>Configuration pour la &amp;recherche</translation>
+        <translation>&amp;Recherche</translation>
     </message>
     <message>
         <source>User manual</source>
@@ -638,6 +705,22 @@ Voulez vous ouvrir le dialogue de paramétrage ?</translation>
         <source>External index dialog</source>
         <translation>Index externes</translation>
     </message>
+    <message>
+        <source>&amp;Erase document history</source>
+        <translation>&amp;Effacer l&apos;historique des documents</translation>
+    </message>
+    <message>
+        <source>First page</source>
+        <translation>Premiere page</translation>
+    </message>
+    <message>
+        <source>Go to first page of results</source>
+        <translation>Aller a la premiere page de resultats</translation>
+    </message>
+    <message>
+        <source>&amp;Indexing configuration</source>
+        <translation>Configuration &amp;Indexation</translation>
+    </message>
 </context>
 <context>
     <name>RclResList</name>
@@ -1237,7 +1320,7 @@ Peut ralentir l&apos;affichage si les documents sont gros.</translation>
     </message>
     <message>
         <source>(no stemming)</source>
-        <translation>(pas d&apos;expansion)</translation>
+        <translation type="obsolete">(pas d&apos;expansion)</translation>
     </message>
     <message>
         <source>The selected directory does not appear to be a Xapian index</source>
@@ -1304,7 +1387,7 @@ Peut ralentir l&apos;affichage si les documents sont gros.</translation>
     </message>
     <message>
         <source>Show document type icons in result list.</source>
-        <translation>Afficher les icônes de type de fichier dans la liste de résultats.</translation>
+        <translation type="obsolete">Afficher les icônes de type de fichier dans la liste de résultats.</translation>
     </message>
     <message>
         <source>Auto-start simple search on whitespace entry.</source>
@@ -1430,7 +1513,7 @@ Peut ralentir l&apos;affichage si les documents sont gros.</translation>
     </message>
     <message>
         <source>Defines the format for each result list paragraph. Use qt html format and printf-like replacements:&lt;br&gt;%A Abstract&lt;br&gt; %D Date&lt;br&gt; %K Keywords (if any)&lt;br&gt; %L Preview and Edit links&lt;br&gt; %M Mime type&lt;br&gt; %N Result number&lt;br&gt; %R Relevance percentage&lt;br&gt; %S Size information&lt;br&gt; %T Title&lt;br&gt; %U Url&lt;br&gt;</source>
-        <translation>Définit the format pour chaque paragraphe de la liste de résultats. Utilise le format html qt et des remplacements à la printf:&lt;br&gt;%A Résumé&lt;br&gt; %D Date&lt;br&gt; %K Mots clefs (s&apos;il y en a)&lt;br&gt; %L Liens aperçu et édition&lt;br&gt; %M Type Mime&lt;br&gt; %N Numéro de résultat&lt;br&gt; %R Pertinence&lt;br&gt; %S Taille&lt;br&gt; %T Titre&lt;br&gt; %U Url&lt;br&gt;</translation>
+        <translation type="obsolete">Définit the format pour chaque paragraphe de la liste de résultats. Utilise le format html qt et des remplacements à la printf:&lt;br&gt;%A Résumé&lt;br&gt; %D Date&lt;br&gt; %K Mots clefs (s&apos;il y en a)&lt;br&gt; %L Liens aperçu et édition&lt;br&gt; %M Type Mime&lt;br&gt; %N Numéro de résultat&lt;br&gt; %R Pertinence&lt;br&gt; %S Taille&lt;br&gt; %T Titre&lt;br&gt; %U Url&lt;br&gt;</translation>
     </message>
     <message>
         <source>Automatically add phrase to simple searches</source>
@@ -1482,12 +1565,40 @@ Ceci devrait donner une meilleure pertinence aux résultats où les termes reche
         <source>Remove from list. This has no effect on the disk index.</source>
         <translation>Oter de la liste. Sans effet sur les données stockées.</translation>
     </message>
+    <message>
+        <source>Remember sorting preference between invocations.</source>
+        <translation type="obsolete">Mémoriser l&apos;état des paramètres de tri.</translation>
+    </message>
+    <message>
+        <source>Defines the format for each result list paragraph. Use qt html format and printf-like replacements:&lt;br&gt;%A Abstract&lt;br&gt; %D Date&lt;br&gt; %I Icon image name&lt;br&gt; %K Keywords (if any)&lt;br&gt; %L Preview and Edit links&lt;br&gt; %M Mime type&lt;br&gt; %N Result number&lt;br&gt; %R Relevance percentage&lt;br&gt; %S Size information&lt;br&gt; %T Title&lt;br&gt; %U Url&lt;br&gt;</source>
+        <translation>Definit le format des paragraphes de la liste de resultats. Utilise le format html qt et des directives de substitution de type printf:&lt;br&gt;%A Abstract&lt;br&gt; %D Date&lt;br&gt; %I Icon image name&lt;br&gt; %K Keywords (if any)&lt;br&gt; %L Preview and Edit links&lt;br&gt; %M Mime type&lt;br&gt; %N Result number&lt;br&gt; %R Relevance percentage&lt;br&gt; %S Size information&lt;br&gt; %T Title&lt;br&gt; %U Url&lt;br&gt;</translation>
+    </message>
+    <message>
+        <source>Remember sort activation state.</source>
+        <translation>Memoriser l&apos;etat d&apos;activation du tri.</translation>
+    </message>
+    <message>
+        <source>Texts beyond over this size will not be highlighted in preview (too slow).</source>
+        <translation type="obsolete">Les textes plus longs ne seront pas surlignes avant previsualisation (trop lent)</translation>
+    </message>
+    <message>
+        <source>Maximum text size highlighted for preview (megabytes)</source>
+        <translation>Taille maximum des textes surlignes avant previsualisation (MO)</translation>
+    </message>
+    <message>
+        <source>Texts over this size will not be highlighted in preview (too slow).</source>
+        <translation>Les texte plus gros ne seront pas surlignes dans la previsualisation (trop lent).</translation>
+    </message>
 </context>
 <context>
     <name>ViewAction</name>
     <message>
         <source>Changing actions with differentcurrent values</source>
-        <translation>Changement d&apos;action avec des valeurs actuelles différentes</translation>
+        <translation type="obsolete">Changement d&apos;action avec des valeurs actuelles différentes</translation>
+    </message>
+    <message>
+        <source>Changing actions with different current values</source>
+        <translation>Changement d&apos;actions avec des valeurs actuelles differentes</translation>
     </message>
 </context>
 <context>
@@ -1608,4 +1719,128 @@ Ceci devrait donner une meilleure pertinence aux résultats où les termes reche
         <translation type="obsolete">Impossible d&apos;éxécuter une requête purement négative. Essayez d&apos;entrer des termes courants dans un champ &apos;ou&apos;</translation>
     </message>
 </context>
+<context>
+    <name>confgui::ConfIndexW</name>
+    <message>
+        <source>Can&apos;t write configuration file</source>
+        <translation>Impossible d&apos;ecrire le fichier de configuration</translation>
+    </message>
+</context>
+<context>
+    <name>confgui::ConfParamFNW</name>
+    <message>
+        <source>Browse</source>
+        <translation>Parcourir</translation>
+    </message>
+</context>
+<context>
+    <name>confgui::ConfParamSLW</name>
+    <message>
+        <source>+</source>
+        <translation>+</translation>
+    </message>
+    <message>
+        <source>-</source>
+        <translation>-</translation>
+    </message>
+</context>
+<context>
+    <name>confgui::ConfSubPanelW</name>
+    <message>
+        <source>Global</source>
+        <translation>Global</translation>
+    </message>
+</context>
+<context>
+    <name>confgui::ConfTopPanelW</name>
+    <message>
+        <source>Top directories</source>
+        <translation>Repertoires de depart</translation>
+    </message>
+    <message>
+        <source>The list of directories where recursive indexing starts. Default: your home.</source>
+        <translation>La liste des repertoires ou l&apos;indexation recursive demarre. Default: votre repertoire par defaut.</translation>
+    </message>
+    <message>
+        <source>Skipped paths</source>
+        <translation>Chemins ignores</translation>
+    </message>
+    <message>
+        <source>These are names of directories which indexing will not enter.&lt;br&gt; May contain wildcards. Must match the paths seen by the indexer (ie: if topdirs includes &apos;/home/me&apos; and &apos;/home&apos; is actually a link to &apos;/usr/home&apos;, a correct skippedPath entry would be &apos;/home/tmp*&apos;, not &apos;/usr/home/tmp*&apos;)</source>
+        <translation>Ce sont des noms de repertoires ou l&apos;indexation n&apos;entrera pas.&lt;br&gt;Ils peuvent contenir des caracteres jokers. Les chemins doivent correspondre a ceux vus par l&apos;indexeur (par exemple: si un des repertoires de depart est &apos;/home/me&apos; et que &apos;/home&apos; est un lien sur &apos;/usr/home&apos;, une entree correcte ici serait &apos;/home/me/tmp*&apos; , pas &apos;/usr/home/me/tmp*&apos;)</translation>
+    </message>
+    <message>
+        <source>Stemming languages</source>
+        <translation>Langue pour l&apos;expansion des termes</translation>
+    </message>
+    <message>
+        <source>The languages for which stemming expansion&lt;br&gt;dictionaries will be built.</source>
+        <translation>Les langages pour lesquels les dictionnaires d&apos;expansion&lt;br&gt;des termes  seront construits.</translation>
+    </message>
+    <message>
+        <source>Log file name</source>
+        <translation>Nom du fichier journal</translation>
+    </message>
+    <message>
+        <source>The file where the messages will be written.&lt;br&gt;Use &apos;stderr&apos; for terminal output</source>
+        <translation>Le nom du fichier ou les messages seront ecrits.&lt;br&gt;Utiliser &apos;stderr&apos; pour le terminal</translation>
+    </message>
+    <message>
+        <source>Log verbosity level</source>
+        <translation>Niveau de verbosite</translation>
+    </message>
+    <message>
+        <source>This value adjusts the amount of messages,&lt;br&gt;from only errors to a lot of debugging data.</source>
+        <translation>Cette valeur ajuste la quantite de messages emis,&lt;br&gt;depuis uniquement les erreurs jusqu&apos;a beaucoup de donnees de debug.</translation>
+    </message>
+    <message>
+        <source>Index flush megabytes interval</source>
+        <translation>Intervalle d&apos;ecriture de l&apos;index en megaoctets</translation>
+    </message>
+    <message>
+        <source>This value adjust the amount of data which is indexed betweeen flushes to disk.&lt;br&gt;This helps control the indexer memory usage. Default 10MB </source>
+        <translation>Ajuste la quantite de donnees lues entre les ecritures sur disque.&lt;br&gt;Controle l&apos;utilisation de la memoire. Defaut 10 Mo </translation>
+    </message>
+    <message>
+        <source>Max disk occupation (%)</source>
+        <translation>Occupation disque maximum (%)</translation>
+    </message>
+    <message>
+        <source>This is the percentage of disk occupation where indexing will fail and stop (to avoid filling up your disk).&lt;br&gt;0 means no limit (this is the default).</source>
+        <translation>Niveau d&apos;occupation du disque ou l&apos;indexation s&apos;arrete (pour eviter un remplissage excessif).&lt;br&gt;0 signifie pas de limite (defaut).</translation>
+    </message>
+    <message>
+        <source>No aspell usage</source>
+        <translation>Pas d&apos;utilisation d&apos;aspell</translation>
+    </message>
+    <message>
+        <source>Disables use of aspell to generate spelling approximation in the term explorer tool.&lt;br&gt; Useful is aspell is absent or does not work. </source>
+        <translation>Interdit l&apos;utilisation d&apos;aspell pour generer un dictionnaire d&apos;approximation phonetique (outil d&apos;exploration des termes)&lt;br&gt;
+Utile si aspell est absent ou ne fonctionne pas bien.</translation>
+    </message>
+    <message>
+        <source>Aspell language</source>
+        <translation>Langue pour aspell</translation>
+    </message>
+    <message>
+        <source>The language for the aspell dictionary. This should look like &apos;en&apos; or &apos;fr&apos; ...&lt;br&gt;If this value is not set, the NLS environment will be used to compute it, which usually works.To get an idea of what is installed on your system, type &apos;aspell config&apos; and look for .dat files inside the &apos;data-dir&apos; directory. </source>
+        <translation>Langue du dictionnaire aspell. Une valeur correcte ressemble a &apos;en&apos; ou &apos;fr&apos;... &lt;br&gt;Si cette valeur n&apos;est pas positionnee, l&apos;environnement est utilise pour la calculer, ce qui marche bien, habituellement. Utiliser &apos;aspell config&apos; et regarder les fichiers .dat dans le repertoire &apos;data-dir&apos; pour connaitre les langues aspell installees sur votre systeme.</translation>
+    </message>
+    <message>
+        <source>Database directory name</source>
+        <translation>Repertoire de stockage de l&apos;index</translation>
+    </message>
+    <message>
+        <source>The name for a directory where to store the index&lt;br&gt;A non-absolute path is taken relative to the  configuration directory. The default is &apos;xapiandb&apos;.</source>
+        <translation>Le nom d&apos;un repertoire ou l&apos;index sera stocke&lt;br&gt;Un chemin relatif demarrera au repertoire de configuration. Le defaut est &apos;xapiandb&apos;.</translation>
+    </message>
+    <message>
+        <source>Use system&apos;s &apos;file&apos; command</source>
+        <translation>Utiliser la commande &apos;file&apos; du systeme</translation>
+    </message>
+    <message>
+        <source>Use the system&apos;s &apos;file&apos; command if internal&lt;br&gt;mime type identification fails.</source>
+        <translation>Utiliser la commande &apos;file&apos; si la determination&lt;br&gt;interne du type mime n&apos;obtient pas de resultat.</translation>
+    </message>
+</context>
 </TS>
