@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: main.cpp,v 1.58 2007/01/08 15:21:32 dockes Exp $ (C) 2005 J.F.Dockes";
+static char rcsid[] = "@(#$Id: main.cpp,v 1.64 2007/11/21 14:15:48 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -57,6 +57,8 @@ static char rcsid[] = "@(#$Id: main.cpp,v 1.58 2007/01/08 15:21:32 dockes Exp $
 #ifdef RCL_USE_ASPELL
 #include "rclaspell.h"
 #endif
+#include "smallut.h"
+#include "recollq.h"
 
 #ifdef WITH_KDE
 static const char description[] =
@@ -77,8 +79,14 @@ Rcl::Db *rcldb;
 Aspell *aspell;
 #endif
 
+RclConfig* RclConfig::getMainConfig()
+{
+    return rclconfig;
+}
+
 RclHistory *g_dynconf;
 int recollNeedsExit;
+int startIndexingAfterConfig;
 static string dbdir;
 static RclMain *mainWindow;
 static string recollsharedir;
@@ -101,7 +109,8 @@ bool maybeOpenDb(string &reason, bool force)
 	LOGDEB(("main: adding [%s]\n", it->c_str()));
 	rcldb->addQueryDb(*it);
     }
-    if (!rcldb->isopen() && !rcldb->open(dbdir, Rcl::Db::DbRO, qopts)) {
+    if (!rcldb->isopen() && !rcldb->open(dbdir, rclconfig->getStopfile(),
+					 Rcl::Db::DbRO, qopts)) {
 	reason = "Could not open database in " + 
 	    dbdir + " wait for indexing to complete?";
 	return false;
@@ -117,20 +126,17 @@ static void recollCleanup()
     LOGDEB2(("recollCleanup: stopping idx thread\n"));
     stop_idxthread();
     LOGDEB2(("recollCleanup: closing database\n"));
-    delete rcldb;
-    rcldb = 0;
-    delete rclconfig;
-    rclconfig = 0;
+    deleteZ(rcldb);
+    deleteZ(rclconfig);
 #ifdef RCL_USE_ASPELL
-    delete aspell;
-    aspell = 0;
+    deleteZ(aspell);
 #endif
     LOGDEB2(("recollCleanup: done\n"));
 }
 
 static void sigcleanup(int)
 {
-    // fprintf(stderr, "sigcleanup\n");
+    fprintf(stderr, "sigcleanup called\n");
     // Cant call exit from here, because the atexit cleanup does some
     // thread stuff that we can't do from signal context.
     // Just set a flag and let the watchdog timer do the work
@@ -140,15 +146,35 @@ static void sigcleanup(int)
 extern void qInitImages_recoll();
 
 static const char *thisprog;
+
+// ATTENTION A LA COMPATIBILITE AVEC LES OPTIONS DE recollq
 static int    op_flags;
-#define OPT_MOINS 0x1
 #define OPT_h     0x4 
 #define OPT_c     0x20
+#define OPT_q     0x40
+#define OPT_o     0x80
+#define OPT_l     0x100
+#define OPT_f     0x200
+#define OPT_a     0x400
+#define OPT_t     0x800
+
 static const char usage [] =
 "\n"
-"recoll [-h] [-c <configdir>]\n"
-"   -h : Print help and exit\n"
-"   -c <configdir> : specify config directory, overriding $RECOLL_CONFDIR\n"
+"recoll [-h] [-c <configdir>] [-q options]\n"
+"  -h : Print help and exit\n"
+"  -c <configdir> : specify config directory, overriding $RECOLL_CONFDIR\n"
+"  [-o|l|f|a] [-t] -q 'query' : search query to be executed as if entered\n"
+"      into simple search. The default is to interpret the argument as a \n"
+"      query language string (but see modifier options)\n"
+"      In most cases, the query string should be quoted with single-quotes to\n"
+"      avoid shell interpretation\n"
+"     -a : the query will be interpreted as an AND query.\n"
+"     -o : the query will be interpreted as an OR query.\n"
+"     -f : the query will be interpreted as a filename search\n"
+"     -l : the query will be interpreted as a query language string (default)\n"
+"     -t : terminal display: no gui. Results go to stdout. MUST be given\n"
+"          explicitely as -t (not ie, -at), and -q <query> MUST\n"
+"          be last on the command line if this is used.\n"
 ;
 static void
 Usage(void)
@@ -160,6 +186,12 @@ Usage(void)
 
 int main(int argc, char **argv)
 {
+    for (int i = 0; i < argc; i++) {
+	if (!strcmp(argv[i], "-t")) {
+	    exit(recollq(&rclconfig, argc, argv));
+	}
+    }
+
 #ifdef WITH_KDE
     KAboutData about("recoll", I18N_NOOP("Recoll"), rclversion, description,
                      KAboutData::License_GPL, "(C) 2006 Jean-Francois Dockes", 0, 0, "jean-francois.dockes@wanadoo.fr");
@@ -174,6 +206,8 @@ int main(int argc, char **argv)
 
     //    fprintf(stderr, "Application created\n");
     string a_config;
+    string qstring;
+
     thisprog = argv[0];
     argc--; argv++;
 
@@ -183,10 +217,18 @@ int main(int argc, char **argv)
 	    Usage();
 	while (**argv)
 	    switch (*(*argv)++) {
+	    case 'a': op_flags |= OPT_a; break;
 	    case 'c':	op_flags |= OPT_c; if (argc < 2)  Usage();
 		a_config = *(++argv);
 		argc--; goto b1;
+	    case 'f': op_flags |= OPT_f; break;
 	    case 'h': op_flags |= OPT_h; Usage();break;
+	    case 'l': op_flags |= OPT_l; break;
+	    case 'o': op_flags |= OPT_o; break;
+	    case 'q':	op_flags |= OPT_q; if (argc < 2)  Usage();
+		qstring = *(++argv);
+		argc--; goto b1;
+	    case 't': op_flags |= OPT_t; break;
 	    }
     b1: argc--; argv++;
     }
@@ -279,8 +321,8 @@ int main(int argc, char **argv)
 
     rcldb = new Rcl::Db;
 
+    bool needindexconfig = false;
     if (!maybeOpenDb(reason)) {
-	startindexing = 1;
 
 	switch (QMessageBox::
 #if (QT_VERSION >= 0x030200)
@@ -296,20 +338,27 @@ int main(int argc, char **argv)
 			       "Click Cancel if you want to edit the configuration file before indexation starts, or Ok to let it proceed."),
 		 "Ok", "Cancel", 0,   0)) {
 
-	case 0: // Ok
+	case 0: // Ok: indexing is going to start.
+	    startindexing = 1;
 	    break;
 
 	case 1: // Cancel
-	    exit(0);
+	    needindexconfig = true;
+	    startindexing = 0;
+	    break;
 	}
     }
 
     mainWindow->show();
-
-    if (prefs.startWithAdvSearchOpen)
-	mainWindow->showAdvSearchDialog();
-    if (prefs.startWithSortToolOpen)
-	mainWindow->showSortDialog();
+    if (needindexconfig) {
+	startIndexingAfterConfig = 1;
+	mainWindow->showIndexConfig();
+    } else {
+	if (prefs.startWithAdvSearchOpen)
+	    mainWindow->showAdvSearchDialog();
+	if (prefs.startWithSortToolOpen)
+	    mainWindow->showSortDialog();
+    }
 
     // Connect exit handlers etc.. Beware, apparently this must come
     // after mainWindow->show() , else the QMessageBox above never
@@ -322,7 +371,22 @@ int main(int argc, char **argv)
     app.connect(&app, SIGNAL(aboutToQuit()), mainWindow, SLOT(close()));
 
     start_idxthread(*rclconfig);
-
+    if (op_flags & OPT_q) {
+	SSearch::SSearchType stype;
+	if (op_flags & OPT_o) {
+	    stype = SSearch::SST_ANY;
+	} else if (op_flags & OPT_f) {
+	    stype = SSearch::SST_FNM;
+	} else if (op_flags & OPT_a) {
+	    stype = SSearch::SST_ALL;
+	} else {
+	    stype = SSearch::SST_LANG;
+	}
+	mainWindow->sSearch->searchTypCMB->setCurrentItem(int(stype));
+	mainWindow->
+	    sSearch->setSearchString(QString::fromUtf8(qstring.c_str()));
+	QTimer::singleShot(0, mainWindow->sSearch, SLOT(startSimpleSearch()));
+    }
     //    fprintf(stderr, "Go\n");
     // Let's go
     return app.exec();
