@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: preview_w.cpp,v 1.17 2007/02/19 18:15:14 dockes Exp $ (C) 2005 J.F.Dockes";
+static char rcsid[] = "@(#$Id: preview_w.cpp,v 1.31 2007/11/15 18:44:51 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -35,10 +35,12 @@ using std::pair;
 #if (QT_VERSION < 0x040000)
 #include <qtextedit.h>
 #include <qprogressdialog.h>
+#define THRFINISHED finished
 #else
 #include <q3textedit.h>
 #include <q3progressdialog.h>
 #include <q3stylesheet.h>
+#define THRFINISHED isFinished
 #endif
 #include <qevent.h>
 #include <qlabel.h>
@@ -47,7 +49,8 @@ using std::pair;
 #include <qlayout.h>
 #include <qtooltip.h>
 #include <qwhatsthis.h>
-#include "qapplication.h"
+#include <qapplication.h>
+#include <qclipboard.h>
 
 #include "debuglog.h"
 #include "pathut.h"
@@ -64,8 +67,113 @@ using std::pair;
 #define wasCanceled wasCancelled
 #endif
 
+#if (QT_VERSION < 0x040000)
+#include <qtextedit.h>
+#include <private/qrichtext_p.h>
+#define QTEXTEDIT QTextEdit
+#define QTEXTCURSOR QTextCursor
+#define QTEXTPARAGRAPH QTextParagraph
+#define QTEXTSTRINGCHAR QTextStringChar
+#else
+#include <q3textedit.h>
+#include "q3richtext_p.h"
+#define QTEXTEDIT Q3TextEdit
+#define QTEXTCURSOR Q3TextCursor
+#define QTEXTPARAGRAPH Q3TextParagraph
+#define QTEXTSTRINGCHAR Q3TextStringChar
+#endif
+// QTextEdit's scrollToAnchor() is supposed to make the anchor visible, but
+// actually, it only moves to the top of the paragraph containing the anchor.
+// As we only have one paragraph, this doesnt' help a lot (qt3 and qt4)
+//
+// So, had to write a different function, inspired from what 
+// qtextedit::find() does, instead. This ones actually moves the
+// cursor, which is probably not necessary, but does what we need.
+//
+// Problem is, it uses the sem-private qrichtext_p.h, which is not
+// even installed under qt4. We use a local copy, which is not nice.
+void QTextEditFixed::moveToAnchor(const QString& name)
+{
+    if (name.isEmpty())
+	return;
+    sync();
+    QTEXTCURSOR l_cursor(document());
+    QTEXTPARAGRAPH* last = document()->lastParagraph();
+    for (;;) {
+	QTEXTSTRINGCHAR* c = l_cursor.paragraph()->at(l_cursor.index());
+	if(c->isAnchor()) {
+	    QString a = c->anchorName();
+	    if ( a == name ||
+		 (a.contains( '#' ) && 
+		  QStringList::split('#', a).contains(name))) {
+		
+		*(textCursor())  = l_cursor;
+		ensureCursorVisible();
+		break;
+	    }
+	}
+	if (l_cursor.paragraph() == last && l_cursor.atParagEnd())
+	    break;
+	l_cursor.gotoNextLetter();
+    }
+}
+
 void Preview::init()
 {
+    setName("Preview");
+    setSizePolicy( QSizePolicy((QSizePolicy::SizeType)5, 
+			       (QSizePolicy::SizeType)5, 0, 0, 
+			       sizePolicy().hasHeightForWidth()));
+    QVBoxLayout* previewLayout =
+	new QVBoxLayout( this, 4, 6, "previewLayout"); 
+
+    pvTab = new QTabWidget(this, "pvTab");
+
+    // Create the first tab. Should be possible to use addEditorTab
+    // but this causes a pb with the sizeing
+    QWidget *unnamed = new QWidget(pvTab, "unnamed");
+    QVBoxLayout *unnamedLayout = 
+	new QVBoxLayout(unnamed, 0, 6, "unnamedLayout"); 
+    QTextEditFixed *pvEdit = new QTextEditFixed(unnamed, "pvEdit");
+    pvEdit->setReadOnly(TRUE);
+    pvEdit->setUndoRedoEnabled(FALSE);
+    unnamedLayout->addWidget(pvEdit);
+    pvTab->insertTab(unnamed, QString::fromLatin1(""));
+    m_tabData.push_back(TabData(pvTab->currentPage()));
+
+    previewLayout->addWidget(pvTab);
+
+    // Create the buttons and entry field
+    QHBoxLayout *layout3 = new QHBoxLayout(0, 0, 6, "layout3"); 
+    searchLabel = new QLabel(this, "searchLabel");
+    layout3->addWidget(searchLabel);
+    searchTextLine = new QLineEdit(this, "searchTextLine");
+    layout3->addWidget(searchTextLine);
+    nextButton = new QPushButton(this, "nextButton");
+    nextButton->setEnabled(TRUE);
+    layout3->addWidget(nextButton);
+    prevButton = new QPushButton(this, "prevButton");
+    prevButton->setEnabled(TRUE);
+    layout3->addWidget(prevButton);
+    clearPB = new QPushButton(this, "clearPB");
+    clearPB->setEnabled(FALSE);
+    layout3->addWidget(clearPB);
+    matchCheck = new QCheckBox(this, "matchCheck");
+    layout3->addWidget(matchCheck);
+
+    previewLayout->addLayout(layout3);
+
+    resize(QSize(640, 480).expandedTo(minimumSizeHint()));
+
+    // buddies
+    searchLabel->setBuddy(searchTextLine);
+
+    searchLabel->setText(tr("&Search for:"));
+    nextButton->setText(tr("&Next"));
+    prevButton->setText(tr("&Previous"));
+    clearPB->setText(tr("Clear"));
+    matchCheck->setText(tr("Match &Case"));
+
 #if 0
     // Couldn't get a small button really in the corner. stays on the left of
     // the button area and looks ugly
@@ -88,37 +196,40 @@ void Preview::init()
 	    this, SLOT(currentChanged(QWidget *)));
     connect(bt, SIGNAL(clicked()), this, SLOT(closeCurrentTab()));
 
-    searchTextLine->installEventFilter(this);
-    pvEdit->installEventFilter(this);
-    dynSearchActive = false;
-    canBeep = true;
-    tabData.push_back(TabData(pvTab->currentPage()));
-    currentW = 0;
+    m_dynSearchActive = false;
+    m_canBeep = true;
+    m_currentW = 0;
     if (prefs.pvwidth > 100) {
 	resize(prefs.pvwidth, prefs.pvheight);
     }
     m_loading = false;
-}
-
-void Preview::destroy()
-{
+    currentChanged(pvTab->currentPage());
+    m_justCreated = true;
+    m_haveAnchors = false;
+    m_curAnchor = 1;
 }
 
 void Preview::closeEvent(QCloseEvent *e)
 {
+    LOGDEB(("Preview::closeEvent. m_loading %d\n", m_loading));
+    if (m_loading) {
+	CancelCheck::instance().setCancel();
+	e->ignore();
+	return;
+    }
     prefs.pvwidth = width();
     prefs.pvheight = height();
-    emit previewExposed(m_searchId, -1);
-    emit previewClosed((QWidget *)this);
+    emit previewExposed(this, m_searchId, -1);
+    emit previewClosed(this);
     QWidget::closeEvent(e);
 }
 
 bool Preview::eventFilter(QObject *target, QEvent *event)
 {
     if (event->type() != QEvent::KeyPress) 
-	return QWidget::eventFilter(target, event);
+	return false;
     
-    LOGDEB(("Preview::eventFilter: keyEvent\n"));
+    LOGDEB1(("Preview::eventFilter: keyEvent\n"));
     QKeyEvent *keyEvent = (QKeyEvent *)event;
     if (keyEvent->key() == Qt::Key_Q && 
 	(keyEvent->state() & Qt::ControlButton)) {
@@ -132,21 +243,21 @@ bool Preview::eventFilter(QObject *target, QEvent *event)
 	// LOGDEB(("Preview::eventFilter: got Shift-Up\n"));
 	TabData *d = tabDataForCurrent();
 	if (d) 
-	    emit(showNext(m_searchId, d->docnum));
+	    emit(showNext(this, m_searchId, d->docnum));
 	return true;
     } else if (keyEvent->key() == Qt::Key_Up &&
 	       (keyEvent->state() & Qt::ShiftButton)) {
 	// LOGDEB(("Preview::eventFilter: got Shift-Down\n"));
 	TabData *d = tabDataForCurrent();
 	if (d) 
-	    emit(showPrev(m_searchId, d->docnum));
+	    emit(showPrev(this, m_searchId, d->docnum));
 	return true;
     } else if (keyEvent->key() == Qt::Key_W &&
 	       (keyEvent->state() & Qt::ControlButton)) {
 	// LOGDEB(("Preview::eventFilter: got ^W\n"));
 	closeCurrentTab();
 	return true;
-    } else if (dynSearchActive) {
+    } else if (m_dynSearchActive) {
 	if (keyEvent->key() == Qt::Key_F3) {
 	    doSearch(searchTextLine->text(), true, false);
 	    return true;
@@ -155,49 +266,56 @@ bool Preview::eventFilter(QObject *target, QEvent *event)
 	    return QApplication::sendEvent(searchTextLine, event);
     } else {
 	QWidget *tw = pvTab->currentPage();
-	QWidget *e = 0;
+	QTextEditFixed *e = 0;
 	if (tw)
-	    e = (QWidget *)tw->child("pvEdit");
+	    e = (QTextEditFixed *)tw->child("pvEdit");
 	LOGDEB1(("Widget: %p, edit %p, target %p\n", tw, e, target));
-	if (e && target == tw && keyEvent->key() == Qt::Key_Slash) {
-	    searchTextLine->setFocus();
-	    dynSearchActive = true;
-	    return true;
+	if (e && target == e) {
+	    if (keyEvent->key() == Qt::Key_Slash) {
+		searchTextLine->setFocus();
+		m_dynSearchActive = true;
+		return true;
+	    } else if (keyEvent->key() == Qt::Key_Space) {
+		e->scrollBy(0, e->visibleHeight());
+		return true;
+	    } else if (keyEvent->key() == Qt::Key_BackSpace) {
+		e->scrollBy(0, -e->visibleHeight());
+		return true;
+	    }
 	}
     }
 
-    return QWidget::eventFilter(target, event);
+    return false;
 }
 
 void Preview::searchTextLine_textChanged(const QString & text)
 {
     LOGDEB1(("search line text changed. text: '%s'\n", text.ascii()));
     if (text.isEmpty()) {
-	dynSearchActive = false;
-	nextButton->setEnabled(false);
-	prevButton->setEnabled(false);
+	m_dynSearchActive = false;
+	//	nextButton->setEnabled(false);
+	//	prevButton->setEnabled(false);
 	clearPB->setEnabled(false);
     } else {
-	dynSearchActive = true;
-	nextButton->setEnabled(true);
-	prevButton->setEnabled(true);
+	m_dynSearchActive = true;
+	//	nextButton->setEnabled(true);
+	//	prevButton->setEnabled(true);
 	clearPB->setEnabled(true);
 	doSearch(text, false, false);
     }
 }
 
 #if (QT_VERSION >= 0x040000)
-#define QTextEdit Q3TextEdit
 #define QProgressDialog Q3ProgressDialog
 #define QStyleSheetItem Q3StyleSheetItem
 #endif
 
-QTextEdit *Preview::getCurrentEditor()
+QTextEditFixed *Preview::getCurrentEditor()
 {
     QWidget *tw = pvTab->currentPage();
-    QTextEdit *edit = 0;
+    QTextEditFixed *edit = 0;
     if (tw) {
-	edit = (QTextEdit*)tw->child("pvEdit");
+	edit = (QTextEditFixed*)tw->child("pvEdit");
     }
     return edit;
 }
@@ -206,72 +324,83 @@ QTextEdit *Preview::getCurrentEditor()
 // current search, trying to advance and possibly wrapping around. If next is
 // false, the search string has been modified, we search for the new string, 
 // starting from the current position
-void Preview::doSearch(const QString &text, bool next, bool reverse, 
+void Preview::doSearch(const QString &_text, bool next, bool reverse, 
 		       bool wordOnly)
 {
-    LOGDEB1(("Preview::doSearch: [%s] next %d rev %d\n", 
-	    (const char *)text.utf8(), int(next), int(reverse)));
-    if (text.isEmpty())
-	return;
-    QTextEdit *edit = getCurrentEditor();
+    LOGDEB(("Preview::doSearch: text [%s] txtlen %d next %d rev %d word %d\n", 
+	    (const char *)_text.utf8(), _text.length(), int(next), 
+	    int(reverse), int(wordOnly)));
+    QString text = _text;
+
+    bool matchCase = matchCheck->isChecked();
+    QTextEditFixed *edit = getCurrentEditor();
     if (edit == 0) {
 	// ??
 	return;
     }
-    bool matchCase = matchCheck->isChecked();
-    int mspara, msindex, mepara, meindex;
-    edit->getSelection(&mspara, &msindex, &mepara, &meindex);
-    if (mspara == -1)
-	mspara = msindex = mepara = meindex = 0;
 
-    if (next) {
-	// We search again, starting from the current match
-	if (reverse) {
-	    // when searching backwards, have to move back one char
-	    if (msindex > 0)
-		msindex --;
-	    else if (mspara > 0) {
-		mspara --;
-		msindex = edit->paragraphLength(mspara);
-	    }
-	} else {
-	    // Forward search: start from end of selection
-	    mspara = mepara;
-	    msindex = meindex;
-	    LOGDEB1(("New para: %d index %d\n", mspara, msindex));
-	}
+    if (text.isEmpty()) {
+	if (m_haveAnchors == false)
+	    return;
+	if (m_curAnchor == m_lastAnchor)
+	    m_curAnchor = 1;
+	else
+	    m_curAnchor++;
+	QString aname = 
+	    QString::fromUtf8(termAnchorName(m_curAnchor).c_str());
+	edit->moveToAnchor(aname);
+	return;
     }
 
-    bool found = edit->find(text, matchCase, wordOnly, 
-			      !reverse, &mspara, &msindex);
-    LOGDEB(("Found at para: %d index %d\n", mspara, msindex));
+    // If next is false, the user added characters to the current
+    // search string.  We need to reset the cursor position to the
+    // start of the previous match, else incremental search is going
+    // to look for the next occurrence instead of trying to lenghten
+    // the current match
+    if (!next) {
+	int ps, is, pe, ie;
+	edit->getSelection(&ps, &is, &pe, &ie);
+	if (is > 0)
+	    is--;
+	else if (ps > 0)
+	    ps--;
+	LOGDEB(("Preview::doSearch: setting cursor to %d %d\n", ps, is));
+	edit->setCursorPosition(ps, is);
+    }
 
-    if (!found && next && true) { // need a 'canwrap' test here
+    LOGDEB(("Preview::doSearch: first find call\n"));
+    bool found = edit->find(text, matchCase, wordOnly, !reverse, 0, 0);
+    LOGDEB(("Preview::doSearch: first find call return\n"));
+    // If not found, try to wrap around. 
+    if (!found && next) { 
+	LOGDEB(("Preview::doSearch: wrapping around\n"));
+	int mspara, msindex;
 	if (reverse) {
 	    mspara = edit->paragraphs();
 	    msindex = edit->paragraphLength(mspara);
 	} else {
 	    mspara = msindex = 0;
 	}
-	found = edit->find(text, matchCase, false, !reverse, &mspara, &msindex);
+	LOGDEB(("Preview::doSearch: 2nd find call\n"));
+	found = edit->find(text,matchCase, false, !reverse, &mspara, &msindex);
+	LOGDEB(("Preview::doSearch: 2nd find call return\n"));
     }
 
     if (found) {
-	canBeep = true;
+	m_canBeep = true;
     } else {
-	if (canBeep)
+	if (m_canBeep)
 	    QApplication::beep();
-	canBeep = false;
+	m_canBeep = false;
     }
+    LOGDEB(("Preview::doSearch: return\n"));
 }
 
-
 void Preview::nextPressed()
 {
     doSearch(searchTextLine->text(), true, false);
 }
 
-
 void Preview::prevPressed()
 {
     doSearch(searchTextLine->text(), true, true);
@@ -281,28 +410,68 @@ void Preview::prevPressed()
 void Preview::currentChanged(QWidget * tw)
 {
     QWidget *edit = (QWidget *)tw->child("pvEdit");
-    currentW = tw;
+    m_currentW = tw;
     LOGDEB1(("Preview::currentChanged(). Editor: %p\n", edit));
     
     if (edit == 0) {
 	LOGERR(("Editor child not found\n"));
-    } else {
-	tw->installEventFilter(this);
-	edit->installEventFilter(this);
-	edit->setFocus();
-	connect(edit, SIGNAL(doubleClicked(int, int)), 
-		this, SLOT(textDoubleClicked(int, int)));
-	TabData *d = tabDataForCurrent();
-	if (d) 
-	    emit(previewExposed(m_searchId, d->docnum));
+	return;
+    }
+    edit->setFocus();
+    // Connect doubleclick but cleanup first just in case this was
+    // already connected.
+    disconnect(edit, SIGNAL(doubleClicked(int, int)), this, 0);
+    connect(edit, SIGNAL(doubleClicked(int, int)), 
+	    this, SLOT(textDoubleClicked(int, int)));
+#if (QT_VERSION >= 0x040000)
+    connect(edit, SIGNAL(selectionChanged()), this, SLOT(selecChanged()));
+#endif
+    tw->installEventFilter(this);
+    edit->installEventFilter(this);
+    TabData *d = tabDataForCurrent();
+    if (d) 
+	emit(previewExposed(this, m_searchId, d->docnum));
+}
+
+#if (QT_VERSION >= 0x040000)
+// I have absolutely no idea why this nonsense is needed to get
+// q3textedit to copy to x11 primary selection when text is
+// selected. This used to be automatic, and, looking at the code, it
+// should happen inside q3textedit (the code here is copied from the
+// private copyToClipboard method). To be checked again with a later
+// qt version.
+void Preview::selecChanged()
+{
+    LOGDEB1(("Selection changed\n"));
+    if (!m_currentW)
+	return;
+    QTextEditFixed *edit = (QTextEditFixed*)m_currentW->child("pvEdit");
+    if (edit == 0) {
+	LOGERR(("Editor child not found\n"));
+	return;
+    }
+    QClipboard *clipboard = QApplication::clipboard();
+    if (edit->hasSelectedText()) {
+	LOGDEB1(("Copying [%s] to primary selection.Clipboard sel supp: %d\n", 
+		(const char *)edit->selectedText().ascii(),
+		clipboard->supportsSelection()));
+        disconnect(QApplication::clipboard(), SIGNAL(selectionChanged()), 
+		   edit, 0);
+	clipboard->setText(edit->selectedText(), QClipboard::Selection);
+        connect(QApplication::clipboard(), SIGNAL(selectionChanged()),
+		edit, SLOT(clipboardChanged()));
     }
 }
+#else 
+void Preview::selecChanged(){}
+#endif
 
 void Preview::textDoubleClicked(int, int)
 {
-    if (!currentW)
+    LOGDEB2(("Preview::textDoubleClicked\n"));
+    if (!m_currentW)
 	return;
-    QTextEdit *edit = (QTextEdit *)currentW->child("pvEdit");
+    QTextEditFixed *edit = (QTextEditFixed *)m_currentW->child("pvEdit");
     if (edit == 0) {
 	LOGERR(("Editor child not found\n"));
 	return;
@@ -313,18 +482,21 @@ void Preview::textDoubleClicked(int, int)
 
 void Preview::closeCurrentTab()
 {
-    if (m_loading)
+    LOGDEB(("Preview::closeCurrentTab: m_loading %d\n", m_loading));
+    if (m_loading) {
+	CancelCheck::instance().setCancel();
 	return;
+    }
     if (pvTab->count() > 1) {
 	QWidget *tw = pvTab->currentPage();
 	if (!tw) 
 	    return;
 	pvTab->removePage(tw);
 	// Have to remove from tab data list
-	for (list<TabData>::iterator it = tabData.begin(); 
-	     it != tabData.end(); it++) {
+	for (list<TabData>::iterator it = m_tabData.begin(); 
+	     it != m_tabData.end(); it++) {
 	    if (it->w == tw) {
-		tabData.erase(it);
+		m_tabData.erase(it);
 		return;
 	    }
 	}
@@ -333,26 +505,29 @@ void Preview::closeCurrentTab()
     }
 }
 
-
-QTextEdit *Preview::addEditorTab()
+QTextEditFixed *Preview::addEditorTab()
 {
     QWidget *anon = new QWidget((QWidget *)pvTab);
     QVBoxLayout *anonLayout = new QVBoxLayout(anon, 1, 1, "anonLayout"); 
-    QTextEdit *editor = new QTextEdit(anon, "pvEdit");
+    QTextEditFixed *editor = new QTextEditFixed(anon, "pvEdit");
     editor->setReadOnly(TRUE);
     editor->setUndoRedoEnabled(FALSE );
     anonLayout->addWidget(editor);
     pvTab->addTab(anon, "Tab");
     pvTab->showPage(anon);
-    tabData.push_back(TabData(anon));
+    m_tabData.push_back(TabData(anon));
     return editor;
 }
 
 void Preview::setCurTabProps(const string &fn, const Rcl::Doc &doc,
 			     int docnum)
 {
-    QString title = QString::fromUtf8(doc.title.c_str(), 
-				      doc.title.length());
+    QString title;
+    map<string,string>::const_iterator meta_it;
+    if ((meta_it = doc.meta.find("title")) != doc.meta.end()) {
+	    title = QString::fromUtf8(meta_it->second.c_str(), 
+				      meta_it->second.length());
+    }
     if (title.length() > 20) {
 	title = title.left(10) + "..." + title.right(10);
     }
@@ -372,12 +547,12 @@ void Preview::setCurTabProps(const string &fn, const Rcl::Doc &doc,
     printableUrl(doc.url, url);
     string tiptxt = url + string("\n");
     tiptxt += doc.mimetype + " " + string(datebuf) + "\n";
-    if (!doc.title.empty())
-	tiptxt += doc.title + "\n";
+    if (meta_it != doc.meta.end() && !meta_it->second.empty())
+	tiptxt += meta_it->second + "\n";
     pvTab->setTabToolTip(w,QString::fromUtf8(tiptxt.c_str(), tiptxt.length()));
 
-    for (list<TabData>::iterator it = tabData.begin(); 
-	 it != tabData.end(); it++) {
+    for (list<TabData>::iterator it = m_tabData.begin(); 
+	 it != m_tabData.end(); it++) {
 	if (it->w == w) {
 	    it->fn = fn;
 	    it->ipath = doc.ipath;
@@ -392,8 +567,8 @@ TabData *Preview::tabDataForCurrent()
     QWidget *w = pvTab->currentPage();
     if (w == 0)
 	return 0;
-    for (list<TabData>::iterator it = tabData.begin(); 
-	 it != tabData.end(); it++) {
+    for (list<TabData>::iterator it = m_tabData.begin(); 
+	 it != m_tabData.end(); it++) {
 	if (it->w == w) {
 	    return &(*it);
 	}
@@ -401,11 +576,12 @@ TabData *Preview::tabDataForCurrent()
     return 0;
 }
 
-bool Preview::makeDocCurrent(const string &fn, const Rcl::Doc &doc)
+bool Preview::makeDocCurrent(const string &fn, size_t sz, 
+			     const Rcl::Doc& doc, int docnum, bool sametab)
 {
-    LOGDEB(("Preview::makeFileCurrent: %s\n", fn.c_str()));
-    for (list<TabData>::iterator it = tabData.begin(); 
-	 it != tabData.end(); it++) {
+    LOGDEB(("Preview::makeDocCurrent: %s\n", fn.c_str()));
+    for (list<TabData>::iterator it = m_tabData.begin(); 
+	 it != m_tabData.end(); it++) {
 	LOGDEB2(("Preview::makeFileCurrent: compare to w %p, file %s\n", 
 		 it->w, it->fn.c_str()));
 	if (!it->fn.compare(fn) && !it->ipath.compare(doc.ipath)) {
@@ -413,7 +589,16 @@ bool Preview::makeDocCurrent(const string &fn, const Rcl::Doc &doc)
 	    return true;
 	}
     }
-    return false;
+    // if just created the first tab was created during init
+    if (!sametab && !m_justCreated && !addEditorTab()) {
+	return false;
+    }
+    m_justCreated = false;
+    if (!loadFileInCurrentTab(fn, sz, doc, docnum)) {
+	closeCurrentTab();
+	return false;
+    }
+    return true;
 }
 
 /*
@@ -504,23 +689,23 @@ class LoadThread : public QThread {
 class ToRichThread : public QThread {
     string &in;
     const HiliteData &hdata;
-    QString &out;
+    list<string> &out;
     int loglevel;
+    int *lastanchor;
  public:
-    ToRichThread(string &i, const HiliteData& hd, QString &o) 
-	: in(i), hdata(hd), out(o)
+    ToRichThread(string &i, const HiliteData& hd, list<string> &o, 
+		 int *lsta)
+	: in(i), hdata(hd), out(o), lastanchor(lsta)
     {
 	    loglevel = DebugLog::getdbl()->getlevel();
     }
     virtual void run()
     {
 	DebugLog::getdbl()->setloglevel(loglevel);
-	string rich;
 	try {
-	    plaintorich(in, rich, hdata, false, true);
+	    plaintorich(in, out, hdata, false, lastanchor);
 	} catch (CancelExcept) {
 	}
-	out = QString::fromUtf8(rich.c_str(), rich.length());
     }
 };
 
@@ -542,7 +727,7 @@ class LoadGuard {
     bool *m_bp;
 public:
     LoadGuard(bool *bp) {m_bp = bp ; *m_bp = true;}
-    ~LoadGuard() {*m_bp = false;}
+    ~LoadGuard() {*m_bp = false; CancelCheck::instance().setCancel(false);}
 };
 
 bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
@@ -554,12 +739,14 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     }
 
     LoadGuard guard(&m_loading);
+    CancelCheck::instance().setCancel(false);
+
+    m_haveAnchors = false;
 
     Rcl::Doc doc = idoc;
-    bool cancel = false;
 
-    if (doc.title.empty()) 
-	doc.title = path_getsimple(doc.url);
+    if (doc.meta["title"].empty()) 
+	doc.meta["title"] = path_getsimple(doc.url);
 
     setCurTabProps(fn, doc, docnum);
 
@@ -586,23 +773,22 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     for (prog = 1;;prog++) {
 	waiter.start();
 	waiter.wait();
-#if (QT_VERSION < 0x040000)
-	if (lthr.finished())
+	if (lthr.THRFINISHED ())
 	    break;
-#else
-	if (lthr.isFinished())
-	    break;
-#endif
 	progress.setProgress(prog , prog <= nsteps-1 ? nsteps : prog+1);
 	qApp->processEvents();
 	if (progress.wasCanceled()) {
 	    CancelCheck::instance().setCancel();
-	    cancel = true;
 	}
 	if (prog >= 5)
 	    sleep(1);
     }
-    if (cancel)
+
+    LOGDEB(("LoadFileInCurrentTab: after file load: cancel %d status %d"
+	    " text length %d\n", 
+	    CancelCheck::instance().cancelState(), status, fdoc.text.length()));
+
+    if (CancelCheck::instance().cancelState())
 	return false;
     if (status != 0) {
 	QString explain;
@@ -620,49 +806,69 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     // Reset config just in case.
     rclconfig->setKeyDir("");
 
-    // Create preview text: highlight search terms (if not too big):
-    QString richTxt;
-    bool highlightTerms = fdoc.text.length() < 1000 *1024;
+    // Create preview text: highlight search terms
+    // We don't do the highlighting for very big texts: too long. We
+    // should at least do special char escaping, in case a '&' or '<'
+    // somehow slipped through previous processing.
+    bool highlightTerms = fdoc.text.length() < 
+	(unsigned long)prefs.maxhltextmbs * 1024 * 1024;
+    // Final text is produced in chunks so that we can display the top
+    // while still inserting at bottom
+    list<QString> qrichlst;
+
     if (highlightTerms) {
 	progress.setLabelText(tr("Creating preview text"));
-	ToRichThread rthr(fdoc.text, m_hData, richTxt);
+	qApp->processEvents();
+	list<string> richlst;
+	ToRichThread rthr(fdoc.text, m_hData, richlst, &m_lastAnchor);
 	rthr.start();
 
 	for (;;prog++) {
 	    waiter.start();	waiter.wait();
-#if (QT_VERSION < 0x040000)
-	if (rthr.finished())
-	    break;
-#else
-	if (rthr.isFinished())
-	    break;
-#endif
+	    if (rthr.THRFINISHED ())
+		break;
 	    progress.setProgress(prog , prog <= nsteps-1 ? nsteps : prog+1);
 	    qApp->processEvents();
 	    if (progress.wasCanceled()) {
 		CancelCheck::instance().setCancel();
-		cancel = true;
 	    }
 	    if (prog >= 5)
 		sleep(1);
 	}
-	if (cancel) {
-	    if (richTxt.length() == 0) {
+
+	// Conversion to rich text done
+	if (CancelCheck::instance().cancelState()) {
+	    if (richlst.size() == 0 || richlst.front().length() == 0) {
 		// We cant call closeCurrentTab here as it might delete
 		// the object which would be a nasty surprise to our
 		// caller.
 		return false;
 	    } else {
-		richTxt += "<b>Cancelled !</b>";
+		richlst.back() += "<b>Cancelled !</b>";
 	    }
 	}
+	// Convert to QString list
+	for (list<string>::iterator it = richlst.begin(); 
+	     it != richlst.end(); it++) {
+	    qrichlst.push_back(QString::fromUtf8(it->c_str(), it->length()));
+	}
     } else {
-	richTxt = fdoc.text.c_str();
+	// No plaintorich() call.
+	// In this case, the text will no be identified as
+	// richtxt/html (no <html> or <qt> etc. at the beginning), and
+	// there is no need to escape special characters.
+	// Also we need to split in chunks (so that the top is displayed faster),
+	// and we must do it on a QString (to avoid utf8 issues).
+	QString qr = QString::fromUtf8(fdoc.text.c_str(), fdoc.text.length());
+	int l = 0;
+	for (int pos = 0; pos < (int)qr.length(); pos += l) {
+	    l = MIN(CHUNKL, qr.length() - pos);
+	    qrichlst.push_back(qr.mid(pos, l));
+	}
     }
-
+	    
     // Load into editor
-    // Do it in several chunks 
-    QTextEdit *editor = getCurrentEditor();
+    QTextEditFixed *editor = getCurrentEditor();
     editor->setText("");
     if (highlightTerms) {
 	QStyleSheetItem *item = 
@@ -674,54 +880,49 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     prog = 2 * nsteps / 3;
     progress.setLabelText(tr("Loading preview text into editor"));
     qApp->processEvents();
-    int l = 0;
-    for (int pos = 0; pos < (int)richTxt.length(); pos += l, prog++) {
+    int instep = 0;
+    for (list<QString>::iterator it = qrichlst.begin(); 
+	 it != qrichlst.end(); it++, prog++, instep++) {
 	progress.setProgress(prog , prog <= nsteps-1 ? nsteps : prog+1);
 	qApp->processEvents();
-	
-	l = MIN(CHUNKL, richTxt.length() - pos);
-	// Avoid breaking inside a tag. Our tags are short (ie: <br>)
-	if (pos + l != (int)richTxt.length()) {
-	    for (int i = -15; i < 0; i++) {
-		if (richTxt[pos+l+i] == '<') {
-		    l = l+i;
-		    break;
-		}
-	    }
-	}
-	editor->append(richTxt.mid(pos, l));
+
+	editor->append(*it);
+
 	// Stay at top
-	if (pos < 5) {
+	if (instep < 5) {
 	    editor->setCursorPosition(0,0);
 	    editor->ensureCursorVisible();
 	}
 
 	if (progress.wasCanceled()) {
-	    cancel = true;
             editor->append("<b>Cancelled !</b>");
-	    LOGDEB(("Cancelled\n"));
+	    LOGDEB(("LoadFileInCurrentTab: cancelled in editor load\n"));
 	    break;
 	}
     }
 
+    progress.close();
+
+    m_haveAnchors = m_lastAnchor != 0;
     if (searchTextLine->text().length() != 0) {
-	canBeep = true;
+	// If there is a current search string, perform the search
+	m_canBeep = true;
 	doSearch(searchTextLine->text(), true, false);
     } else {
-	QString aname = QString::fromUtf8(firstTermAnchorName);
-	LOGDEB2(("Calling scrolltoanchor [%s]\n", (const char *)aname.utf8()));
-	editor->scrollToAnchor(aname);
-#ifdef QT_SCROLL_TO_ANCHOR_BUG
-	bool wasC = matchCheck->isChecked();
-	matchCheck->setChecked(false);
-	bool ocanbeep = canBeep;
-	canBeep = false;
-	doSearch(QString::fromUtf8(firstTermBeacon), 0, false, false);
-	canBeep = ocanbeep;
-	editor->del();
-	matchCheck->setChecked(wasC);
-#endif
+	// Position to the first query term
+	if (m_haveAnchors) {
+	    QString aname = QString::fromUtf8(termAnchorName(1).c_str());
+	    LOGDEB2(("Call movetoanchor(%s)\n", (const char *)aname.utf8()));
+	    editor->moveToAnchor(aname);
+	    m_curAnchor = 1;
+	}
     }
-    emit(previewExposed(m_searchId, docnum));
+
+    // Enter document in document history
+    g_dynconf->enterDoc(fn, doc.ipath);
+
+    editor->setFocus();
+    emit(previewExposed(this, m_searchId, docnum));
+    LOGDEB(("LoadFileInCurrentTab: returning true\n"));
     return true;
 }
