@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: rclmain_w.cpp,v 1.25 2007/02/14 10:10:43 dockes Exp $ (C) 2005 J.F.Dockes";
+static char rcsid[] = "@(#$Id: rclmain_w.cpp,v 1.45 2007/11/21 14:15:48 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@ using std::pair;
 #include <qmessagebox.h>
 #if (QT_VERSION < 0x040000)
 #include <qcstring.h>
+#include <qpopupmenu.h>
 #endif
 #include <qtabwidget.h>
 #include <qtimer.h>
@@ -50,6 +51,8 @@ using std::pair;
 #include <qpushbutton.h>
 #include <qimage.h>
 #include <qiconset.h>
+#include <qapplication.h>
+#include <qcursor.h>
 
 #include "recoll.h"
 #include "debuglog.h"
@@ -69,11 +72,14 @@ using std::pair;
 #include "internfile.h"
 #include "docseqdb.h"
 #include "docseqhist.h"
+#include "confguiindex.h"
+using namespace confgui;
 
 #include "rclmain_w.h"
 #include "moc_rclmain_w.cpp"
 
 extern "C" int XFlush(void *);
+QString g_stringAllStem, g_stringNoStem;
 
 // Taken from qt designer. Don't know why it's needed.
 #if (QT_VERSION < 0x040000)
@@ -94,6 +100,7 @@ void RclMain::init()
     asearchform = 0;
     sortform = 0;
     uiprefs = 0;
+    indexConfig = 0;
     spellform = 0;
     m_searchId = 0;
     // Set the focus to the search terms entry:
@@ -104,12 +111,52 @@ void RclMain::init()
 	QFont nfont(prefs.reslistfontfamily, prefs.reslistfontsize);
 	resList->setFont(nfont);
     }
+
+    // Stemming language menu
+    g_stringNoStem = tr("(no stemming)");
+    g_stringAllStem = tr("(all languages)");
+    m_idNoStem = preferencesMenu->insertItem(g_stringNoStem);
+    m_stemLangToId[g_stringNoStem] = m_idNoStem;
+    m_idAllStem = preferencesMenu->insertItem(g_stringAllStem);
+    m_stemLangToId[g_stringAllStem] = m_idAllStem;
+
+    // Can't get the stemming languages from the db at this stage as
+    // db not open yet (the case where it does not even exist makes
+    // things complicated). So get the languages from the config
+    // instead
+    string slangs;
+    list<string> langs;
+    if (rclconfig->getConfParam("indexstemminglanguages", slangs)) {
+	stringToStrings(slangs, langs);
+    } else {
+	QMessageBox::warning(0, "Recoll", 
+			     tr("error retrieving stemming languages"));
+    }
+    int curid = prefs.queryStemLang == "ALL" ? m_idAllStem : m_idNoStem;
+    int id; 
+    for (list<string>::const_iterator it = langs.begin(); 
+	 it != langs.end(); it++) {
+	QString qlang = QString::fromAscii(it->c_str(), it->length());
+	id = preferencesMenu->insertItem(qlang);
+	m_stemLangToId[qlang] = id;
+	if (prefs.queryStemLang == qlang) {
+	    curid = id;
+	}
+    }
+    preferencesMenu->setItemChecked(curid, true);
+
+    // Connections
     connect(sSearch, SIGNAL(startSearch(RefCntr<Rcl::SearchData>)), 
 		this, SLOT(startSearch(RefCntr<Rcl::SearchData>)));
-
+    connect(preferencesMenu, SIGNAL(activated(int)),
+	    this, SLOT(setStemLang(int)));
+    connect(preferencesMenu, SIGNAL(aboutToShow()),
+	    this, SLOT(adjustPrefsMenu()));
     // signals and slots connections
     connect(sSearch, SIGNAL(clearSearch()),
 	    this, SLOT(resetSearch()));
+    connect(firstPageAction, SIGNAL(activated()), 
+	    resList, SLOT(resultPageFirst()));
     connect(prevPageAction, SIGNAL(activated()), 
 	    resList, SLOT(resultPageBack()));
     connect(nextPageAction, SIGNAL(activated()),
@@ -135,6 +182,8 @@ void RclMain::init()
     connect(fileExitAction, SIGNAL(activated() ), this, SLOT(fileExit() ) );
     connect(fileStart_IndexingAction, SIGNAL(activated()), 
 	    this, SLOT(startIndexing()));
+    connect(fileEraseDocHistoryAction, SIGNAL(activated()), 
+	    this, SLOT(eraseDocHistory()));
     connect(helpAbout_RecollAction, SIGNAL(activated()), 
 	    this, SLOT(showAboutDialog()));
     connect(userManualAction, SIGNAL(activated()), this, SLOT(startManual()));
@@ -151,14 +200,15 @@ void RclMain::init()
     toolsSpellAction->setEnabled(FALSE);
 #endif
 
-    connect(preferencesQuery_PrefsAction, SIGNAL(activated()), 
-	    this, SLOT(showUIPrefs()));
+    connect(indexConfigAction, SIGNAL(activated()), this, SLOT(showIndexConfig()));
+    connect(queryPrefsAction, SIGNAL(activated()), this, SLOT(showUIPrefs()));
     connect(extIdxAction, SIGNAL(activated()), this, SLOT(showExtIdxDialog()));
 
 
 #if (QT_VERSION < 0x040000)
     nextPageAction->setIconSet(createIconSet("nextpage.png"));
     prevPageAction->setIconSet(createIconSet("prevpage.png"));
+    firstPageAction->setIconSet(createIconSet("firstpage.png"));
     toolsSpellAction->setIconSet(QPixmap::fromMimeSource("spell.png"));
     toolsDoc_HistoryAction->setIconSet(QPixmap::fromMimeSource("history.png"));
     toolsAdvanced_SearchAction->setIconSet(QPixmap::fromMimeSource("asearch.png"));
@@ -167,98 +217,89 @@ void RclMain::init()
     toolsSpellAction->setIcon(QIcon(":/images/spell.png"));
     nextPageAction->setIcon(QIcon(":/images/nextpage.png"));
     prevPageAction->setIcon(QIcon(":/images/prevpage.png"));
+    firstPageAction->setIcon(QIcon(":/images/firstpage.png"));
     toolsDoc_HistoryAction->setIcon(QIcon(":/images/history.png"));
     toolsAdvanced_SearchAction->setIcon(QIcon(":/images/asearch.png"));
     toolsSort_parametersAction->setIcon(QIcon(":/images/sortparms.png"));
 #endif
-}
 
 
-//#define SHOWEVENTS
-#if defined(SHOWEVENTS)
-static const char *eventTypeToStr(int tp)
-{
-    switch (tp) {
-    case 0: return "None";
-    case 1: return "Timer";
-    case 2: return "MouseButtonPress";
-    case 3: return "MouseButtonRelease";
-    case 4: return "MouseButtonDblClick";
-    case 5: return "MouseMove";
-    case 6: return "KeyPress";
-    case 7: return "KeyRelease";
-    case 8: return "FocusIn";
-    case 9: return "FocusOut";
-    case 10: return "Enter";
-    case 11: return "Leave";
-    case 12: return "Paint";
-    case 13: return "Move";
-    case 14: return "Resize";
-    case 15: return "Create";
-    case 16: return "Destroy";
-    case 17: return "Show";
-    case 18: return "Hide";
-    case 19: return "Close";
-    case 20: return "Quit";
-    case 21: return "Reparent";
-    case 22: return "ShowMinimized";
-    case 23: return "ShowNormal";
-    case 24: return "WindowActivate";
-    case 25: return "WindowDeactivate";
-    case 26: return "ShowToParent";
-    case 27: return "HideToParent";
-    case 28: return "ShowMaximized";
-    case 29: return "ShowFullScreen";
-    case 30: return "Accel";
-    case 31: return "Wheel";
-    case 32: return "AccelAvailable";
-    case 33: return "CaptionChange";
-    case 34: return "IconChange";
-    case 35: return "ParentFontChange";
-    case 36: return "ApplicationFontChange";
-    case 37: return "ParentPaletteChange";
-    case 38: return "ApplicationPaletteChange";
-    case 39: return "PaletteChange";
-    case 40: return "Clipboard";
-    case 42: return "Speech";
-    case 50: return "SockAct";
-    case 51: return "AccelOverride";
-    case 52: return "DeferredDelete";
-    case 60: return "DragEnter";
-    case 61: return "DragMove";
-    case 62: return "DragLeave";
-    case 63: return "Drop";
-    case 64: return "DragResponse";
-    case 70: return "ChildInserted";
-    case 71: return "ChildRemoved";
-    case 72: return "LayoutHint";
-    case 73: return "ShowWindowRequest";
-    case 74: return "WindowBlocked";
-    case 75: return "WindowUnblocked";
-    case 80: return "ActivateControl";
-    case 81: return "DeactivateControl";
-    case 82: return "ContextMenu";
-    case 83: return "IMStart";
-    case 84: return "IMCompose";
-    case 85: return "IMEnd";
-    case 86: return "Accessibility";
-    case 87: return "TabletMove";
-    case 88: return "LocaleChange";
-    case 89: return "LanguageChange";
-    case 90: return "LayoutDirectionChange";
-    case 91: return "Style";
-    case 92: return "TabletPress";
-    case 93: return "TabletRelease";
-    case 94: return "OkRequest";
-    case 95: return "HelpRequest";
-    case 96: return "WindowStateChange";
-    case 97: return "IconDrag";
-    case 1000: return "User";
-    case 65535: return "MaxUser";
-    default: return "Unknown";
+    // If requested by prefs, restore sort state. The easiest way is to let
+    // a SortForm do it for us.
+    if (prefs.keepSort && prefs.sortActive) {
+	SortForm sf(0);
+	connect(&sf, SIGNAL(sortDataChanged(DocSeqSortSpec)), 
+		this, SLOT(sortDataChanged(DocSeqSortSpec)));
+	// Have to call setdata again after sig connected...
+	sf.setData();
     }
 }
-#endif
+
+void RclMain::setStemLang(int id)
+{
+    LOGDEB(("RclMain::setStemLang(%d)\n", id));
+    // Check that the menu entry is for a stemming language change
+    // (might also be "show prefs" etc.
+    bool isLangId = false;
+    for (map<QString, int>::const_iterator it = m_stemLangToId.begin();
+	 it != m_stemLangToId.end(); it++) {
+	if (id == it->second)
+	    isLangId = true;
+    }
+    if (!isLangId)
+	return;
+
+    // Set the "checked" item state for lang entries
+    for (map<QString, int>::const_iterator it = m_stemLangToId.begin();
+	 it != m_stemLangToId.end(); it++) {
+	preferencesMenu->setItemChecked(it->second, false);
+    }
+    preferencesMenu->setItemChecked(id, true);
+
+    // Retrieve language value (also handle special cases), set prefs,
+    // notify that we changed
+    QString lang;
+    if (id == m_idNoStem) {
+	lang = "";
+    } else if (id == m_idAllStem) {
+	lang = "ALL";
+    } else {
+	lang = preferencesMenu->text(id);
+    }
+    prefs.queryStemLang = lang;
+    LOGDEB(("RclMain::setStemLang(%d): lang [%s]\n", 
+	    id, (const char *)prefs.queryStemLang.ascii()));
+    rwSettings(true);
+    emit stemLangChanged(lang);
+}
+
+// Set the checked stemming language item before showing the prefs menu
+void RclMain::setStemLang(const QString& lang)
+{
+    LOGDEB(("RclMain::setStemLang(%s)\n", (const char *)lang.ascii()));
+    int id;
+    if (lang == "") {
+	id = m_idNoStem;
+    } else if (lang == "ALL") {
+	id = m_idAllStem;
+    } else {
+	map<QString, int>::iterator it = m_stemLangToId.find(lang);
+	if (it == m_stemLangToId.end()) 
+	    return;
+	id = it->second;
+    }
+    for (map<QString, int>::const_iterator it = m_stemLangToId.begin();
+	 it != m_stemLangToId.end(); it++) {
+	preferencesMenu->setItemChecked(it->second, false);
+    }
+    preferencesMenu->setItemChecked(id, true);
+}
+
+// Prefs menu about to show
+void RclMain::adjustPrefsMenu()
+{
+    setStemLang(prefs.queryStemLang);
+}
 
 void RclMain::closeEvent( QCloseEvent * )
 {
@@ -363,9 +404,10 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
     LOGDEB(("RclMain::startSearch\n"));
     // The db may have been closed at the end of indexing
     string reason;
-    if (!maybeOpenDb(reason)) {
+    // If indexing is being performed, we reopen the db at each query.
+    if (!maybeOpenDb(reason, indexingdone == 0)) {
 	QMessageBox::critical(0, "Recoll", QString(reason.c_str()));
-	exit(1);
+	return;
     }
 
     resList->resetSearch();
@@ -373,10 +415,17 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
     int qopts = 0;
     if (!prefs.queryStemLang.length() == 0)
 	qopts |= Rcl::Db::QO_STEM;
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-    if (!rcldb->setQuery(sdata, qopts, prefs.queryStemLang.ascii())) {
+    string stemLang = (const char *)prefs.queryStemLang.ascii();
+    if (stemLang == "ALL") {
+	rclconfig->getConfParam("indexstemminglanguages", stemLang);
+    }
+
+    if (!rcldb->setQuery(sdata, qopts, stemLang)) {
 	QMessageBox::warning(0, "Recoll", tr("Cant start query: ") +
 			     QString::fromAscii(rcldb->getReason().c_str()));
+	QApplication::restoreOverrideCursor();
 	return;
     }
     curPreview = 0;
@@ -385,6 +434,7 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
     m_docSource = RefCntr<DocSequence>(src);
     m_searchData = sdata;
     setDocSequence();
+    QApplication::restoreOverrideCursor();
 }
 
 void RclMain::resetSearch()
@@ -456,11 +506,26 @@ void RclMain::showSpellDialog()
 
 }
 
+void RclMain::showIndexConfig()
+{
+    LOGDEB(("showIndexConfig()\n"));
+    if (indexConfig == 0) {
+	indexConfig = new ConfIndexW(0, rclconfig);
+	LOGDEB(("showIndexConfig(): confindexW created\n"));
+    } else {
+	// Close and reopen, in hope that makes us visible...
+	indexConfig->close();
+    }
+    indexConfig->show();
+}
+
 void RclMain::showUIPrefs()
 {
     if (uiprefs == 0) {
 	uiprefs = new UIPrefsDialog(0);
 	connect(uiprefs, SIGNAL(uiprefsDone()), this, SLOT(setUIPrefs()));
+	connect(this, SIGNAL(stemLangChanged(const QString&)), 
+		uiprefs, SLOT(setStemLang(const QString&)));
     } else {
 	// Close and reopen, in hope that makes us visible...
 	uiprefs->close();
@@ -484,9 +549,10 @@ void RclMain::showExtIdxDialog()
 // If a preview (toplevel) window gets closed by the user, we need to
 // clean up because there is no way to reopen it. And check the case
 // where the current one is closed
-void RclMain::previewClosed(QWidget *w)
+void RclMain::previewClosed(Preview *w)
 {
-    if (w == (QWidget *)curPreview) {
+    LOGDEB(("RclMain::previewClosed(%p)\n", w));
+    if (w == curPreview) {
 	LOGDEB(("Active preview closed\n"));
 	curPreview = 0;
     } else {
@@ -525,7 +591,9 @@ void RclMain::startPreview(int docnum, int mod)
 	curPreview = 0;
     }
     if (curPreview == 0) {
-	curPreview = new Preview(0);
+	HiliteData hdata;
+	m_searchData->getTerms(hdata.terms, hdata.groups, hdata.gslks);
+	curPreview = new Preview(m_searchId, hdata);
 	if (curPreview == 0) {
 	    QMessageBox::warning(0, tr("Warning"), 
 				 tr("Can't create preview window"),
@@ -533,32 +601,20 @@ void RclMain::startPreview(int docnum, int mod)
 				 QMessageBox::NoButton);
 	    return;
 	}
-	HiliteData hdata;
-	m_searchData->getTerms(hdata.terms, hdata.groups, hdata.gslks);
-	curPreview->setSId(m_searchId, hdata);
-	curPreview->setCaption(resList->getDescription());
-	connect(curPreview, SIGNAL(previewClosed(QWidget *)), 
-		this, SLOT(previewClosed(QWidget *)));
+	connect(curPreview, SIGNAL(previewClosed(Preview *)), 
+		this, SLOT(previewClosed(Preview *)));
 	connect(curPreview, SIGNAL(wordSelect(QString)),
 		this, SLOT(ssearchAddTerm(QString)));
-	connect(curPreview, SIGNAL(showNext(int, int)),
-		this, SLOT(previewNextInTab(int, int)));
-	connect(curPreview, SIGNAL(showPrev(int, int)),
-		this, SLOT(previewPrevInTab(int, int)));
-	connect(curPreview, SIGNAL(previewExposed(int, int)),
-		this, SLOT(previewExposed(int, int)));
+	connect(curPreview, SIGNAL(showNext(Preview *, int, int)),
+		this, SLOT(previewNextInTab(Preview *, int, int)));
+	connect(curPreview, SIGNAL(showPrev(Preview *, int, int)),
+		this, SLOT(previewPrevInTab(Preview *, int, int)));
+	connect(curPreview, SIGNAL(previewExposed(Preview *, int, int)),
+		this, SLOT(previewExposed(Preview *, int, int)));
+	curPreview->setCaption(resList->getDescription());
 	curPreview->show();
-    } else {
-	if (curPreview->makeDocCurrent(fn, doc)) {
-	    // Already there
-	    return;
-	}
-	(void)curPreview->addEditorTab();
-    }
-    // Enter document in document history
-    g_dynconf->enterDoc(fn, doc.ipath);
-    if (!curPreview->loadFileInCurrentTab(fn, st.st_size, doc, docnum))
-	curPreview->closeCurrentTab();
+    } 
+    curPreview->makeDocCurrent(fn, st.st_size, doc, docnum);
 }
 
 /** 
@@ -578,7 +634,7 @@ void RclMain::startPreview(Rcl::Doc doc)
 			     fn.c_str());
 	return;
     }
-    Preview *preview = new Preview(0);
+    Preview *preview = new Preview(0, HiliteData());
     if (preview == 0) {
 	QMessageBox::warning(0, tr("Warning"), 
 			     tr("Can't create preview window"),
@@ -586,38 +642,52 @@ void RclMain::startPreview(Rcl::Doc doc)
 			     QMessageBox::NoButton);
 	return;
     }
-    preview->setSId(0, HiliteData());
     connect(preview, SIGNAL(wordSelect(QString)),
 	    this, SLOT(ssearchAddTerm(QString)));
-    g_dynconf->enterDoc(fn, doc.ipath);
     preview->show();
-    if (!preview->loadFileInCurrentTab(fn, st.st_size, doc, 0))
-	preview->closeCurrentTab();
+    preview->makeDocCurrent(fn, st.st_size, doc, 0);
 }
 
 // Show next document from result list in current preview tab
-void RclMain::previewNextInTab(int sid, int docnum)
+void RclMain::previewNextInTab(Preview * w, int sid, int docnum)
+{
+    previewPrevOrNextInTab(w, sid, docnum, true);
+}
+
+// Show previous document from result list in current preview tab
+void RclMain::previewPrevInTab(Preview * w, int sid, int docnum)
+{
+    previewPrevOrNextInTab(w, sid, docnum, false);
+}
+
+// Combined next/prev from result list in current preview tab
+void RclMain::previewPrevOrNextInTab(Preview * w, int sid, int docnum, bool nxt)
 {
     LOGDEB(("RclMain::previewNextInTab  sid %d docnum %d, m_sid %d\n", 
 	    sid, docnum, m_searchId));
 
+    if (w == 0) // ??
+	return;
+
     if (sid != m_searchId) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("This search is not active any more"));
 	return;
     }
 
-    docnum++;
-    if (docnum >= resList->getResCnt()) {
+    if (nxt)
+	docnum++;
+    else 
+	docnum--;
+    if (docnum < 0 || docnum >= resList->getResCnt()) {
 	QApplication::beep();
 	return;
     }
 
     Rcl::Doc doc;
     if (!resList->getDoc(docnum, doc)) {
-	QMessageBox::warning(0, "Recoll",
-			     tr("Cannot retrieve document info" 
-				     " from database"));
+	QMessageBox::warning(0, "Recoll", 
+			     tr("Cannot retrieve document info from database"));
 	return;
     }
 	
@@ -630,48 +700,11 @@ void RclMain::previewNextInTab(int sid, int docnum)
 	return;
     }
 
-    if (!curPreview->loadFileInCurrentTab(fn, st.st_size, doc, docnum))
-	curPreview->closeCurrentTab();
-}
-
-// Show previous document from result list in current preview tab
-void RclMain::previewPrevInTab(int sid, int docnum)
-{
-    LOGDEB(("RclMain::previewPrevInTab  sid %d docnum %d, m_sid %d\n", 
-	    sid, docnum, m_searchId));
-
-    if (sid != m_searchId) {
-	QMessageBox::warning(0, "Recoll", 
-			     tr("This search is not active any more"));
-	return;
-    }
-    if (docnum <= 0) {
-	QApplication::beep();
-	return;
-    }
-    docnum--;
-    Rcl::Doc doc;
-    if (!resList->getDoc(docnum, doc)) {
-	QMessageBox::warning(0, "Recoll",
-			     tr("Cannot retrieve document info" 
-				     " from database"));
-	return;
-    }
-	
-    // Check that the file exists in the file system
-    string fn = urltolocalpath(doc.url);
-    struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
-	QMessageBox::warning(0, "Recoll", tr("Cannot access document file: ") +
-			     fn.c_str());
-	return;
-    }
-    if (!curPreview->loadFileInCurrentTab(fn, st.st_size, doc, docnum))
-	curPreview->closeCurrentTab();
+    w->makeDocCurrent(fn, st.st_size, doc, docnum, true);
 }
 
 // Preview tab exposed: possibly tell reslist (to color the paragraph)
-void RclMain::previewExposed(int sid, int docnum)
+void RclMain::previewExposed(Preview *, int sid, int docnum)
 {
     LOGDEB2(("RclMain::previewExposed: sid %d docnum %d, m_sid %d\n", 
 	     sid, docnum, m_searchId));
@@ -683,25 +716,24 @@ void RclMain::previewExposed(int sid, int docnum)
 }
 
 // Add term to simple search. Term comes out of double-click in
-// reslist or preview. The cleanup code is really horrible. Selection
-// out of the reslist will typically look like 
-// <!-- Fragment -->sometext
+// reslist or preview. 
 // It would probably be better to cleanup in preview.ui.h and
 // reslist.cpp and do the proper html stuff in the latter case
-// (which is different because it's format is explicit richtext
+// (which is different because it format is explicit richtext
 // instead of auto as for preview, needed because it's built by
 // fragments?).
-static const char* punct = " \t()<>\"'[]{}!^*,\n\r";
+static const char* punct = " \t()<>\"'[]{}!^*.,:;\n\r";
 void RclMain::ssearchAddTerm(QString term)
 {
+    LOGDEB(("RclMain::ssearchAddTerm: [%s]\n", (const char *)term.utf8()));
     string t = (const char *)term.utf8();
     string::size_type pos = t.find_last_not_of(punct);
     if (pos == string::npos)
 	return;
     t = t.substr(0, pos+1);
-    pos = t.find_last_of(punct);
+    pos = t.find_first_not_of(punct);
     if (pos != string::npos)
-	t = t.substr(pos+1);
+	t = t.substr(pos);
     if (t.empty())
 	return;
     term = QString::fromUtf8(t.c_str());
@@ -883,7 +915,7 @@ void RclMain::showDocHistory()
     string reason;
     if (!maybeOpenDb(reason)) {
 	QMessageBox::critical(0, "Recoll", QString(reason.c_str()));
-	exit(1);
+	return;
     }
     // Construct a bogus SearchData structure
     m_searchData = 
@@ -892,6 +924,7 @@ void RclMain::showDocHistory()
 
     m_searchId++;
 
+    // If you change the title, also change it in eraseDocHistory()
     DocSequenceHistory *src = 
 	new DocSequenceHistory(rcldb, g_dynconf, 
 			       string(tr("Document history").utf8()));
@@ -900,6 +933,16 @@ void RclMain::showDocHistory()
     setDocSequence();
 }
 
+void RclMain::eraseDocHistory()
+{
+    g_dynconf->eraseAll(RclHistory::docSubkey);
+    // We want to reset the displayed history if it is currently shown. Using
+    // the title value is an ugly hack
+    if (m_docSource->title() == 
+	string((const char *)tr("Document history").utf8())) {
+	showDocHistory();
+    }
+}
 
 void RclMain::sortDataChanged(DocSeqSortSpec spec)
 {
@@ -929,5 +972,6 @@ void RclMain::enableNextPage(bool yesno)
 void RclMain::enablePrevPage(bool yesno)
 {
     prevPageAction->setEnabled(yesno);
+    firstPageAction->setEnabled(yesno);
 }
 
