@@ -53,6 +53,12 @@ public: DummyRclMainBase(QWidget *parent) :Q3MainWindow(parent){setupUi(this);}
 #endif
 //MOC_SKIP_END
 
+class Preview;
+namespace confgui {
+class ConfIndexW;
+}
+using confgui::ConfIndexW;
+
 class RclMain : public DummyRclMainBase
 {
     Q_OBJECT
@@ -65,7 +71,6 @@ public:
     }
     ~RclMain() {}
 
-
 public slots:
     virtual bool close();
     virtual void fileExit();
@@ -73,7 +78,7 @@ public slots:
     virtual void startIndexing();
     virtual void startSearch(RefCntr<Rcl::SearchData> sdata);
     virtual void setDocSequence();
-    virtual void previewClosed(QWidget * w);
+    virtual void previewClosed(Preview *w);
     virtual void showAdvSearchDialog();
     virtual void showSortDialog();
     virtual void showSpellDialog();
@@ -83,6 +88,7 @@ public slots:
     virtual void showExtIdxDialog();
     virtual void sortDataChanged(DocSeqSortSpec spec);
     virtual void showUIPrefs();
+    virtual void showIndexConfig();
     virtual void setUIPrefs();
     virtual void enableNextPage(bool);
     virtual void enablePrevPage(bool);
@@ -92,10 +98,19 @@ public slots:
     virtual void startPreview(Rcl::Doc doc);
     virtual void startNativeViewer(int docnum);
     virtual void startNativeViewer(Rcl::Doc doc);
-    virtual void previewNextInTab(int sid, int docnum);
-    virtual void previewPrevInTab(int sid, int docnum);
-    virtual void previewExposed(int sid, int docnum);
+    virtual void previewNextInTab(Preview *, int sid, int docnum);
+    virtual void previewPrevInTab(Preview *, int sid, int docnum);
+    virtual void previewExposed(Preview *, int sid, int docnum);
     virtual void resetSearch();
+    virtual void eraseDocHistory();
+    // Callback for setting the stemming language through the prefs menu
+    virtual void setStemLang(int id);
+    // Prefs menu about to show, set the checked lang entry
+    virtual void adjustPrefsMenu();
+
+signals:
+    void stemLangChanged(const QString& lang);
+
 protected:
     virtual void closeEvent( QCloseEvent * );
 
@@ -104,6 +119,7 @@ private:
     AdvSearch *asearchform;
     SortForm *sortform;
     UIPrefsDialog *uiprefs;
+    ConfIndexW  *indexConfig;
     SpellW *spellform;
 
     RefCntr<Rcl::SearchData> m_searchData;
@@ -114,7 +130,14 @@ private:
     // Serial number of current search for this process.
     // Used to match to preview windows
     int                      m_searchId; 
+    map<QString, int>        m_stemLangToId;
+    int                      m_idNoStem;
+    int                      m_idAllStem;
+
     virtual void init();
+    virtual void previewPrevOrNextInTab(Preview *, int sid, int docnum, 
+					bool next);
+    virtual void setStemLang(const QString& lang);
 };
 
 #endif // RCLMAIN_W_H
