@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: reslist.cpp,v 1.23 2007/02/08 09:03:28 dockes Exp $ (C) 2005 J.F.Dockes";
+static char rcsid[] = "@(#$Id: reslist.cpp,v 1.36 2007/11/15 18:05:00 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 
 #include <time.h>
@@ -20,7 +20,9 @@ static char rcsid[] = "@(#$Id: reslist.cpp,v 1.23 2007/02/08 09:03:28 dockes Exp
 #if (QT_VERSION < 0x040000)
 #include <qpopupmenu.h>
 #else
+#ifndef __APPLE__
 #include <qx11info_x11.h>
+#endif
 #include <q3popupmenu.h>
 #include <q3stylesheet.h>
 #include <q3mimefactory.h>
@@ -29,11 +31,11 @@ static char rcsid[] = "@(#$Id: reslist.cpp,v 1.23 2007/02/08 09:03:28 dockes Exp
 #endif
 
 #include "debuglog.h"
+#include "smallut.h"
 #include "recoll.h"
 #include "guiutils.h"
 #include "pathut.h"
 #include "docseq.h"
-#include "transcode.h"
 #include "pathut.h"
 #include "mimehandler.h"
 #include "plaintorich.h"
@@ -51,7 +53,6 @@ ResList::ResList(QWidget* parent, const char* name)
 {
     if (!name)
 	setName("resList");
-    setTextFormat(Qt::RichText);
     setReadOnly(TRUE);
     setUndoRedoEnabled(FALSE);
     languageChange();
@@ -59,12 +60,14 @@ ResList::ResList(QWidget* parent, const char* name)
     setTabChangesFocus(true);
 
     // signals and slots connections
-    connect(this, SIGNAL(clicked(int, int)), this, SLOT(clicked(int,int)));
     connect(this, SIGNAL(linkClicked(const QString &, int)), 
 	    this, SLOT(linkWasClicked(const QString &, int)));
     connect(this, SIGNAL(headerClicked()), this, SLOT(showQueryDetails()));
     connect(this, SIGNAL(doubleClicked(int,int)), 
 	    this, SLOT(doubleClicked(int, int)));
+#if (QT_VERSION >= 0x040000)
+    connect(this, SIGNAL(selectionChanged()), this, SLOT(selecChanged()));
+#endif
     m_winfirst = -1;
     m_curPvDoc = -1;
     m_lstClckMod = 0;
@@ -79,20 +82,22 @@ extern "C" int XFlush(void *);
 void ResList::resetSearch() 
 {
     m_winfirst = -1;
+    m_curPvDoc = -1;
     // There should be a progress bar for long searches but there isn't 
     // We really want the old result list to go away, otherwise, for a
     // slow search, the user will wonder if anything happened. The
     // following helps making sure that the textedit is really
     // blank. Else, there are often icons or text left around
     clear();
-    append(".");
+    QTEXTBROWSER::append(".");
     clear();
 #if (QT_VERSION < 0x040000)
     XFlush(qt_xdisplay());
 #else
+#ifndef __APPLE__
     XFlush(QX11Info::display());
 #endif
-
+#endif
 }
 
 void ResList::languageChange()
@@ -103,10 +108,8 @@ void ResList::languageChange()
 // Acquire new docsource
 void ResList::setDocSource(RefCntr<DocSequence> docsource)
 {
-    m_winfirst = -1;
+    resetSearch();
     m_docSource = docsource;
-    m_curPvDoc = -1;
-
     resultPageNext();
 }
 
@@ -244,6 +247,13 @@ void ResList::resultPageBack()
     resultPageNext();
 }
 
+// Go to the first page
+void ResList::resultPageFirst()
+{
+    m_winfirst = -1;
+    resultPageNext();
+}
+
 // Convert byte count into unit (KB/MB...) appropriate for display
 static string displayableBytes(long size)
 {
@@ -261,6 +271,18 @@ static string displayableBytes(long size)
     return string(sizebuf);
 }
 
+void ResList::append(const QString &text)
+{
+    QTEXTBROWSER::append(text);
+#if 0
+    {
+	FILE *fp = fopen("/tmp/debugreslist", "a");
+	fprintf(fp, "%s\n", (const char *)text.utf8());
+	fclose(fp);
+    }
+#endif
+}
+
 // Fill up result list window with next screen of hits
 void ResList::resultPageNext()
 {
@@ -386,22 +408,19 @@ void ResList::resultPageNext()
 	if (percent == -1) {
 	    percent = 0;
 	    // Document not available, maybe other further, will go on.
-	    doc.abstract = string(tr("Unavailable document").utf8());
+	    doc.meta["abstract"] = string(tr("Unavailable document").utf8());
 	}
 
 	// Determine icon to display if any
 	string img_name;
-	if (prefs.showicons) {
-	    string iconpath;
-	    string iconname = rclconfig->getMimeIconName(doc.mimetype,
-							 &iconpath);
-	    LOGDEB1(("Img file; %s\n", iconpath.c_str()));
-	    QImage image(iconpath.c_str());
-	    if (!image.isNull()) {
-		img_name = string("img_") + iconname;
-		QMimeSourceFactory::defaultFactory()->
-		    setImage(img_name.c_str(), image);
-	    }
+	string iconpath;
+	string iconname = rclconfig->getMimeIconName(doc.mimetype, &iconpath);
+	LOGDEB1(("Img file; %s\n", iconpath.c_str()));
+	QImage image(iconpath.c_str());
+	if (!image.isNull()) {
+	    img_name = string("img_") + iconname;
+	    QMimeSourceFactory::defaultFactory()->
+		setImage(img_name.c_str(), image);
 	}
 
 	// Percentage of 'relevance'
@@ -413,8 +432,8 @@ void ResList::resultPageNext()
 	printableUrl(doc.url, url);
 
 	// Make title out of file name if none yet
-	if (doc.title.empty()) {
-	    doc.title = path_getsimple(url);
+	if (doc.meta["title"].empty()) {
+	    doc.meta["title"] = path_getsimple(url);
 	}
 
 	// Result number
@@ -456,11 +475,12 @@ void ResList::resultPageNext()
 	    (doc.syntabs || prefs.queryReplaceAbstract)) {
 	    abstract = m_docSource->getAbstract(doc);
 	} else {
-	    abstract = doc.abstract;
+	    abstract = doc.meta["abstract"];
 	}
-
-	string richabst;
-	plaintorich(abstract, richabst, hdata, true);
+	// No need to call escapeHtml(), plaintorich handles it
+	list<string> lr;
+	plaintorich(abstract, lr, hdata, true, 0, 100000);
+	string richabst = lr.front();
 
 	// Links;
 	string linksbuf;
@@ -476,40 +496,37 @@ void ResList::resultPageNext()
 	}
 
 	// Build the result list paragraph:
-	string result;
+	chunk = "";
 
 	// Subheader: this is used by history
 	if (!sh.empty())
-	    result += string("<p><b>") + sh + "</p>\n<p>";
+	    chunk += "<p><b>" + QString::fromUtf8(sh.c_str()) + "</p>\n<p>";
 	else
-	    result += "<p>";
-
-	if (!img_name.empty()) {
-	    result += "<img source=\"" + img_name + "\" align=\"left\">";
-	}
+	    chunk += "<p>";
 
 	// Configurable stuff
 	map<char,string> subs;
 	subs['A'] = !richabst.empty() ? richabst + "<br>" : "";
 	subs['D'] = datebuf;
-	subs['K'] = !doc.keywords.empty() ? doc.keywords + "<br>" : "";
+	subs['I'] = img_name;
+	subs['K'] = !doc.meta["keywords"].empty() ? 
+	    escapeHtml(doc.meta["keywords"]) + "<br>" : "";
 	subs['L'] = linksbuf;
 	subs['N'] = numbuf;
 	subs['M'] = doc.mimetype;
 	subs['R'] = perbuf;
 	subs['S'] = sizebuf;
-	subs['T'] = doc.title;
+	subs['T'] = escapeHtml(doc.meta["title"]);
 	subs['U'] = url;
 
 	string formatted;
 	pcSubst(sformat, formatted, subs);
-	result += formatted;
+	chunk += QString::fromUtf8(formatted.c_str());
 
-	result += "</p>\n";
+	chunk += "</p>\n";
 
-	LOGDEB2(("Chunk: [%s]\n", result.c_str()));
-	QString str = QString::fromUtf8(result.c_str(), result.length());
-	append(str);
+	LOGDEB2(("Chunk: [%s]\n", (const char *)chunk.utf8()));
+	append(chunk);
 	setCursorPosition(0,0);
 	ensureCursorVisible();
 
@@ -530,35 +547,16 @@ void ResList::resultPageNext()
 	    chunk += tr("Next");
 	    chunk += "</b></a>";
 	}
-	chunk += "</p>\n";
-	append(chunk);
     }
-    append("</body></qt>");
+    chunk += "</p>\n";
+    chunk += "</body></qt>\n";
+    append(chunk);
 
     // Possibly color paragraph of current preview if any
     previewExposed(m_curPvDoc);
     ensureCursorVisible();
 }
 
-// Single click in result list use this for document selection, if no
-// text selection active:
-void ResList::clicked(int par, int)
-{
-    LOGDEB2(("ResList::clicked\n"));
-
-    // It's very ennoying, textBrowser always has selected text. This
-    // is bound to change with qt releases!
-    if (hasSelectedText()&& selectedText().compare("<!--StartFragment-->")) {
-	// Give priority to text selection, do nothing
-	LOGDEB2(("%s\n", (const char *)(selectedText().ascii())));
-	return;
-    }
-    LOGDEB(("click at par %d (with %s %s)\n", par, 
-	    (m_lstClckMod & Qt::ControlButton) ? "Ctrl" : "",
-	    (m_lstClckMod & Qt::ShiftButton) ? "Shft" : ""));
-
-}
-
 // Color paragraph (if any) of currently visible preview
 void ResList::previewExposed(int docnum)
 {
@@ -585,6 +583,32 @@ void ResList::previewExposed(int docnum)
     setParagraphBackgroundColor(par, color);
 }
 
+// SELECTION BEWARE: these emit simple text if the list format is
+// Auto. If we get back to using Rich for some reason, there will be
+// need to extract the simple text here.
+
+#if (QT_VERSION >= 0x040000)
+// I have absolutely no idea why this nonsense is needed to get
+// q3textedit to copy to x11 primary selection when text is
+// selected. This used to be automatic, and, looking at the code, it
+// should happen inside q3textedit (the code here is copied from the
+// private copyToClipboard method). To be checked again with a later
+// qt version. Seems to be needed at least up to 4.2.3
+void ResList::selecChanged()
+{
+    QClipboard *clipboard = QApplication::clipboard();
+    if (hasSelectedText()) {
+        disconnect(QApplication::clipboard(), SIGNAL(selectionChanged()), 
+		   this, 0);
+	clipboard->setText(selectedText(), QClipboard::Selection);
+        connect(QApplication::clipboard(), SIGNAL(selectionChanged()),
+		this, SLOT(clipboardChanged()));
+    }
+}
+#else
+void ResList::selecChanged(){}
+#endif
+
 // Double click in res list: add selection to simple search
 void ResList::doubleClicked(int, int)
 {
@@ -594,7 +618,7 @@ void ResList::doubleClicked(int, int)
 
 void ResList::linkWasClicked(const QString &s, int clkmod)
 {
-    LOGDEB(("ResList::linkClicked: [%s]\n", s.ascii()));
+    LOGDEB(("ResList::linkWasClicked: [%s]\n", s.ascii()));
     int i = atoi(s.ascii()+1) -1;
     int what = s.ascii()[0];
     switch (what) {
@@ -622,6 +646,8 @@ RCLPOPUP *ResList::createPopupMenu(const QPoint& pos)
     int para = paragraphAt(pos);
     clicked(para, 0);
     m_popDoc = docnumfromparnum(para);
+    if (m_popDoc < 0) 
+	return 0;
     RCLPOPUP *popup = new RCLPOPUP(this);
     popup->insertItem(tr("&Preview"), this, SLOT(menuPreview()));
     popup->insertItem(tr("&Edit"), this, SLOT(menuEdit()));
