@@ -1,6 +1,6 @@
 #ifndef _RESLIST_H_INCLUDED_
 #define _RESLIST_H_INCLUDED_
-/* @(#$Id: reslist.h,v 1.9 2007/01/19 15:22:50 dockes Exp $  (C) 2005 J.F.Dockes */
+/* @(#$Id: reslist.h,v 1.13 2007/08/02 06:33:35 dockes Exp $  (C) 2005 J.F.Dockes */
 
 #include <list>
 
@@ -38,17 +38,16 @@ class ResList : public QTEXTBROWSER
     virtual bool getDoc(int docnum, Rcl::Doc &);
 
     virtual void setDocSource(RefCntr<DocSequence> source);
-    virtual RCLPOPUP *createPopupMenu(const QPoint& pos);
     virtual QString getDescription(); // Printable actual query performed on db
     virtual int getResCnt(); // Return total result list size
 
  public slots:
     virtual void resetSearch();
-    virtual void clicked(int, int);
     virtual void doubleClicked(int, int);
     virtual void resPageUpOrBack(); // Page up pressed
     virtual void resPageDownOrNext(); // Page down pressed
     virtual void resultPageBack(); // Display previous page of results
+    virtual void resultPageFirst(); // Display first page of results
     virtual void resultPageNext(); // Display next (or first) page of results
     virtual void menuPreview();
     virtual void menuEdit();
@@ -57,6 +56,9 @@ class ResList : public QTEXTBROWSER
     virtual void menuExpand();
     virtual void menuSeeParent();
     virtual void previewExposed(int);
+    virtual void append(const QString &text);
+    // Only used for qt ver >=4 but seems we cant undef it
+    virtual void selecChanged();
 
  signals:
     void nextPageAvailable(bool);
@@ -68,7 +70,7 @@ class ResList : public QTEXTBROWSER
     void headerClicked();
     void docExpand(int);
     void wordSelect(QString);
-    void linkClicked(const QString&, int);
+    void linkClicked(const QString&, int); // See emitLinkClicked()
 
  protected:
     void keyPressEvent(QKeyEvent *e);
@@ -96,6 +98,7 @@ class ResList : public QTEXTBROWSER
     void emitLinkClicked(const QString &s) {
 	emit linkClicked(s, m_lstClckMod);
     };
+    virtual RCLPOPUP *createPopupMenu(const QPoint& pos);
 };
 
 
