@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: uiprefs_w.cpp,v 1.16 2007/02/14 10:10:43 dockes Exp $ (C) 2005 J.F.Dockes";
+static char rcsid[] = "@(#$Id: uiprefs_w.cpp,v 1.22 2007/09/08 17:25:49 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -96,8 +96,7 @@ void UIPrefsDialog::setFromPrefs()
 {
     // Entries per result page spinbox
     pageLenSB->setValue(prefs.respagesize);
-    // Show icons checkbox
-    useIconsCB->setChecked(prefs.showicons);
+    maxHLTSB->setValue(prefs.maxhltextmbs);
     autoSearchCB->setChecked(prefs.autoSearchOnWS);
     syntlenSB->setValue(prefs.syntAbsLen);
     syntctxSB->setValue(prefs.syntAbsCtx);
@@ -105,6 +104,7 @@ void UIPrefsDialog::setFromPrefs()
     initStartAdvCB->setChecked(prefs.startWithAdvSearchOpen);
     initStartSortCB->setChecked(prefs.startWithSortToolOpen);
     useDesktopOpenCB->setChecked(prefs.useDesktopOpen);
+    keepSortCB->setChecked(prefs.keepSort);
 
     // Result list font family and size
     reslistFontFamily = prefs.reslistfontfamily;
@@ -122,25 +122,22 @@ void UIPrefsDialog::setFromPrefs()
 
     // Stemming language combobox
     stemLangCMB->clear();
-    stemLangCMB->insertItem(tr("(no stemming)"));
+    stemLangCMB->insertItem(g_stringNoStem);
+    stemLangCMB->insertItem(g_stringAllStem);
     list<string> langs;
     if (!getStemLangs(langs)) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("error retrieving stemming languages"));
     }
-    int i = 0, cur = -1;
+    int cur = prefs.queryStemLang == ""  ? 0 : 1;
     for (list<string>::const_iterator it = langs.begin(); 
 	 it != langs.end(); it++) {
 	stemLangCMB->
 	    insertItem(QString::fromAscii(it->c_str(), it->length()));
-	i++;
-	if (cur == -1) {
-	    if (!strcmp(prefs.queryStemLang.ascii(), it->c_str()))
-		cur = i;
+	if (cur == 0 && !strcmp(prefs.queryStemLang.ascii(), it->c_str())) {
+	    cur = stemLangCMB->count();
 	}
     }
-    if (cur < 0)
-	cur = 0;
     stemLangCMB->setCurrentItem(cur);
 
     autoPhraseCB->setChecked(prefs.ssearchAutoPhrase);
@@ -170,18 +167,31 @@ void UIPrefsDialog::setFromPrefs()
 
 void UIPrefsDialog::accept()
 {
-    prefs.showicons = useIconsCB->isChecked();
     prefs.autoSearchOnWS = autoSearchCB->isChecked();
     prefs.respagesize = pageLenSB->value();
+    prefs.maxhltextmbs = maxHLTSB->value();
 
     prefs.reslistfontfamily = reslistFontFamily;
     prefs.reslistfontsize = reslistFontSize;
     prefs.reslistformat =  rlfTE->text();
+    // Don't let us set the old default format from here, this would
+    // get reset to the new default. Ugly hack
+    if (prefs.reslistformat == 
+	QString::fromAscii(prefs.getV18DfltResListFormat())) {
+	prefs.reslistformat += " ";
+	rlfTE->setText(prefs.reslistformat);
+    }
+    if (prefs.reslistformat.stripWhiteSpace().isEmpty()) {
+	prefs.reslistformat = prefs.getDfltResListFormat();
+	rlfTE->setText(prefs.reslistformat);
+    }
 
     prefs.htmlBrowser =  helpBrowserLE->text();
 
     if (stemLangCMB->currentItem() == 0) {
 	prefs.queryStemLang = "";
+    } else if (stemLangCMB->currentItem() == 1) {
+	prefs.queryStemLang = "ALL";
     } else {
 	prefs.queryStemLang = stemLangCMB->currentText();
     }
@@ -193,6 +203,7 @@ void UIPrefsDialog::accept()
     prefs.startWithAdvSearchOpen = initStartAdvCB->isChecked();
     prefs.startWithSortToolOpen = initStartSortCB->isChecked();
     prefs.useDesktopOpen = useDesktopOpenCB->isChecked();
+    prefs.keepSort = keepSortCB->isChecked();
 
     prefs.syntAbsLen = syntlenSB->value();
     prefs.syntAbsCtx = syntctxSB->value();
@@ -223,6 +234,24 @@ void UIPrefsDialog::reject()
     QDialog::reject();
 }
 
+void UIPrefsDialog::setStemLang(const QString& lang)
+{
+    int cur = 0;
+    if (lang == "") {
+	cur = 0;
+    } else if (lang == "ALL") {
+	cur = 1;
+    } else {
+	for (int i = 1; i < stemLangCMB->count(); i++) {
+	    if (lang == stemLangCMB->text(i)) {
+		cur = i;
+		break;
+	    }
+	}
+    }
+    stemLangCMB->setCurrentItem(cur);
+}
+
 void UIPrefsDialog::showFontDialog()
 {
     bool ok;
