@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: searchdata.cpp,v 1.13 2007/02/13 10:58:31 dockes Exp $ (C) 2006 J.F.Dockes";
+static char rcsid[] = "@(#$Id: searchdata.cpp,v 1.19 2007/10/04 12:26:04 dockes Exp $ (C) 2006 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,8 @@ static char rcsid[] = "@(#$Id: searchdata.cpp,v 1.13 2007/02/13 10:58:31 dockes
 #include "textsplit.h"
 #include "unacpp.h"
 #include "utf8iter.h"
+#include "stoplist.h"
+#include "rclconfig.h"
 
 #ifndef NO_NAMESPACES
 using namespace std;
@@ -70,10 +72,25 @@ bool SearchData::toNativeQuery(Rcl::Db &db, void *d, const string& stemlang)
 
     // Add the file type filtering clause if any
     if (!m_filetypes.empty()) {
-	list<Xapian::Query> pqueries;
-	Xapian::Query tq;
+	vector<string> exptps;
+	exptps.reserve(m_filetypes.size());
+	// Expand categories
+	RclConfig *cfg = RclConfig::getMainConfig();
 	for (vector<string>::iterator it = m_filetypes.begin(); 
 	     it != m_filetypes.end(); it++) {
+	    if (cfg && cfg->isMimeCategory(*it)) {
+		list<string>tps;
+		cfg->getMimeCatTypes(*it, tps);
+		exptps.insert(exptps.end(), tps.begin(), tps.end());
+	    } else {
+		exptps.push_back(*it);
+	    }
+	}
+	    
+	list<Xapian::Query> pqueries;
+	Xapian::Query tq;
+	for (vector<string>::iterator it = exptps.begin(); 
+	     it != exptps.end(); it++) {
 	    string term = "T" + *it;
 	    LOGDEB(("Adding file type term: [%s]\n", term.c_str()));
 	    tq = tq.empty() ? Xapian::Query(term) : 
@@ -136,12 +153,23 @@ bool SearchData::getTerms(vector<string>& terms,
 // terms and phrases. 
 class wsQData : public TextSplitCB {
  public:
+    wsQData(const StopList &_stops) 
+	: stops(_stops), alltermcount(0)
+    {}
     vector<string> terms;
     bool takeword(const std::string &term, int , int, int) {
+	alltermcount++;
 	LOGDEB1(("wsQData::takeword: %s\n", term.c_str()));
+	if (stops.hasStops() && stops.isStop(term)) {
+	    LOGDEB1(("wsQData::takeword [%s] in stop list\n", term.c_str()));
+	    return true;
+	}
 	terms.push_back(term);
 	return true;
     }
+    const StopList &stops;
+    int alltermcount; // Count of terms including stopwords: this is
+		      // for adjusting phrase/near slack
 };
 
 /** 
@@ -158,10 +186,11 @@ public:
     { }
 
     bool processUserString(const string &iq,
-		   const string &prefix,
-		   string &ermsg,
-		   list<Xapian::Query> &pqueries,
-		   int slack = 0, bool useNear = false);
+			   const string &prefix,
+			   string &ermsg,
+			   list<Xapian::Query> &pqueries, 
+			   const StopList &stops,
+			   int slack = 0, bool useNear = false);
 
     bool getTerms(vector<string>& terms, 
 		  vector<vector<string> >& groups) 
@@ -313,7 +342,9 @@ bool StringToXapianQ::processUserString(const string &iq,
 					const string &prefix,
 					string &ermsg,
 					list<Xapian::Query> &pqueries,
-					int slack, bool useNear)
+					const StopList& stops,
+					int slack, bool useNear
+					)
 {
     LOGDEB(("StringToXapianQ:: query string: [%s]\n", iq.c_str()));
     ermsg.erase();
@@ -335,35 +366,47 @@ bool StringToXapianQ::processUserString(const string &iq,
 	     it != phrases.end(); it++) {
 	    LOGDEB(("strToXapianQ: phrase or word: [%s]\n", it->c_str()));
 
-	    // If there are both spans and single words in this element,
-	    // we need to use a word split, else a phrase query including
-	    // a span would fail if we didn't adjust the proximity to
-	    // account for the additional span term which is complicated.
-	    wsQData splitDataS, splitDataW;
-	    TextSplit splitterS(&splitDataS, (TextSplit::Flags)
-				(TextSplit::TXTS_ONLYSPANS | 
-				 TextSplit::TXTS_KEEPWILD));
+	    // If there are multiple spans in this element, including
+	    // at least one composite, we have to do something
+	    // else a phrase query including a span would fail. 
+	    // Ex: "term0@term1 term2" is onlyspans-split as:
+	    //   0 term0@term1             0   12
+	    //   2 term2                  13   18
+	    // The position of term1 is 2, not 1, so a phrase search
+	    // would fail.
+	    // We used to do  word split, searching for 
+	    // "term0 term01 term1" instead, which may have worse 
+	    // performance, but will succeed.
+	    // We now adjust the phrase/near slack by the term count
+	    // difference (this is mainly better for cjk where this is a very
+	    // common occurrence because of the ngrams thing.
+	    wsQData splitDataS(stops), splitDataW(stops);
+	    TextSplit splitterS(&splitDataS, 
+				TextSplit::Flags(TextSplit::TXTS_ONLYSPANS | 
+						 TextSplit::TXTS_KEEPWILD));
 	    splitterS.text_to_words(*it);
-	    TextSplit splitterW(&splitDataW, (TextSplit::Flags)
-				(TextSplit::TXTS_NOSPANS | 
-				 TextSplit::TXTS_KEEPWILD));
+	    TextSplit splitterW(&splitDataW, 
+				TextSplit::Flags(TextSplit::TXTS_NOSPANS | 
+						 TextSplit::TXTS_KEEPWILD));
 	    splitterW.text_to_words(*it);
 	    wsQData *splitData = &splitDataS;
 	    if (splitDataS.terms.size() > 1 && 
-		splitDataS.terms.size() != splitDataW.terms.size())
-		splitData = &splitDataW;
+		splitDataS.terms.size() != splitDataW.terms.size()) {
+		slack += splitDataW.terms.size() - splitDataS.terms.size();
+		// used to: splitData = &splitDataW;
+	    }
 
 	    LOGDEB(("strToXapianQ: splitter term count: %d\n", 
 		     splitData->terms.size()));
 	    switch (splitData->terms.size()) {
 	    case 0: continue;// ??
 	    case 1: 
-		// Not a real phrase: one term. Still may be expanded
-		// (stem or wildcard)
+		// Just a term. Still may be expanded (by stem or
+		// wildcard) to an OR list.
 		{
 		    string term = splitData->terms.front();
 		    list<string> exp;  
-		    string sterm;
+		    string sterm; // dumb version of user term
 		    stripExpandTerm(false, term, exp, sterm);
 		    m_terms.insert(m_terms.end(), exp.begin(), exp.end());
 		    // Push either term or OR of stem-expanded set
@@ -386,10 +429,11 @@ bool StringToXapianQ::processUserString(const string &iq,
 		break;
 
 	    default:
-		// Phrase/near: transform into a PHRASE or NEAR xapian
-		// query, the element of which can themselves be OR
-		// queries if the terms get expanded by stemming or
-		// wildcards (we don't do stemming for PHRASE though)
+		// Element had several terms: transform into a PHRASE
+		// or NEAR xapian query, the elements of which can
+		// themselves be OR queries if the terms get expanded
+		// by stemming or wildcards (we don't do stemming for
+		// PHRASE though)
 		Xapian::Query::op op = useNear ? Xapian::Query::OP_NEAR : 
 		Xapian::Query::OP_PHRASE;
 		list<Xapian::Query> orqueries;
@@ -418,7 +462,8 @@ bool StringToXapianQ::processUserString(const string &iq,
 		pqueries.push_back(Xapian::Query(op,
 						 orqueries.begin(),
 						 orqueries.end(),
-					 splitData->terms.size() + slack));
+						 splitData->alltermcount 
+						 + slack));
 		// Add NEAR/PHRASE groups to the highlighting data. Must
 		// push all combinations
 		vector<vector<string> > allcombs;
@@ -444,34 +489,6 @@ bool StringToXapianQ::processUserString(const string &iq,
     return true;
 }
 
-// Try to translate field specification into field prefix. This should
-// probably be an Rcl::Db method and much more configurable (store
-// prefix translation list in config ?)
-static string fieldToPrefix(const string& i_field)
-{
-    static map<string, string> fldToPrefs;
-    if (fldToPrefs.empty()) {
-	fldToPrefs["title"] = "S";
-	fldToPrefs["caption"] = "S";
-	fldToPrefs["subject"] = "S";
-
-	fldToPrefs["author"] = "A";
-	fldToPrefs["creator"] = "A";
-	fldToPrefs["from"] = "A";
-
-	fldToPrefs["keyword"] = "K";
-	fldToPrefs["tag"] = "K";
-	fldToPrefs["keywords"] = "K";
-	fldToPrefs["tags"] = "K";
-    }
-    string fld(i_field); 
-    stringtolower(fld);
-    map<string, string>::const_iterator it = fldToPrefs.find(fld);
-    if (it != fldToPrefs.end())
-	return it->second;
-    return "";
-}
-
 static const string nullstemlang;
 
 // Translate a simple OR, AND, or EXCL search clause. 
@@ -498,7 +515,7 @@ bool SearchDataClauseSimple::toNativeQuery(Rcl::Db &db, void *p,
     }
     string prefix;
     if (!m_field.empty())
-	prefix = fieldToPrefix(m_field);
+	db.fieldToPrefix(m_field, prefix);
     list<Xapian::Query> pqueries;
 
     // We normally boost the original term in the stem expansion list. Don't
@@ -508,7 +525,8 @@ bool SearchDataClauseSimple::toNativeQuery(Rcl::Db &db, void *p,
 	(m_parentSearch == 0 && !m_haveWildCards);
 
     StringToXapianQ tr(db, l_stemlang, doBoostUserTerm);
-    if (!tr.processUserString(m_text, prefix, m_reason, pqueries))
+    if (!tr.processUserString(m_text, prefix, m_reason, pqueries, 
+			      db.getStopList()))
 	return false;
     if (pqueries.empty()) {
 	LOGERR(("SearchDataClauseSimple: resolved to null query\n"));
@@ -551,7 +569,7 @@ bool SearchDataClauseDist::toNativeQuery(Rcl::Db &db, void *p,
 
     string prefix;
     if (!m_field.empty())
-	prefix = fieldToPrefix(m_field);
+	db.fieldToPrefix(m_field, prefix);
 
     // We normally boost the original term in the stem expansion list. Don't
     // do it if there are wildcards anywhere, this would skew the results.
@@ -570,7 +588,8 @@ bool SearchDataClauseDist::toNativeQuery(Rcl::Db &db, void *p,
     string s = string("\"") + m_text + string("\"");
     bool useNear = (m_tp == SCLT_NEAR);
     StringToXapianQ tr(db, l_stemlang, doBoostUserTerm);
-    if (!tr.processUserString(s, prefix, m_reason, pqueries, m_slack, useNear))
+    if (!tr.processUserString(s, prefix, m_reason, pqueries, db.getStopList(),
+			      m_slack, useNear))
 	return false;
     if (pqueries.empty()) {
 	LOGERR(("SearchDataClauseDist: resolved to null query\n"));
