@@ -1,6 +1,6 @@
 #ifndef _STEMDB_H_INCLUDED_
 #define _STEMDB_H_INCLUDED_
-/* @(#$Id: stemdb.h,v 1.3 2006/12/19 12:11:21 dockes Exp $  (C) 2004 J.F.Dockes */
+/* @(#$Id: stemdb.h,v 1.4 2007/08/01 10:04:53 dockes Exp $  (C) 2004 J.F.Dockes */
 /// Stem database code
 /// 
 /// Stem databases list stems and the set of index terms they expand to. They 
@@ -30,7 +30,7 @@ extern bool createDb(Xapian::Database& xdb,
 		     const std::string& dbdir, const std::string& lang);
 /// Expand term to stem siblings
 extern bool stemExpand(const std::string& dbdir, 
-		       const std::string& lang,
+		       const std::string& langs,
 		       const std::string& term,
 		       list<string>& result);
 #ifndef NO_NAMESPACES
