@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: fstreewalk.cpp,v 1.11 2007/02/02 10:12:58 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: fstreewalk.cpp,v 1.14 2007/08/30 09:01:52 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -65,6 +65,13 @@ FsTreeWalker::~FsTreeWalker()
     delete data;
 }
 
+void FsTreeWalker::setOpts(Options opts)
+{
+    if (data) {
+	data->options = opts;
+    }
+}
+
 string FsTreeWalker::getReason()
 {
     return data->reason.str();
@@ -89,6 +96,17 @@ bool FsTreeWalker::setSkippedNames(const list<string> &patterns)
     data->skippedNames.unique();
     return true;
 }
+bool FsTreeWalker::inSkippedNames(const string& name)
+{
+    list<string>::const_iterator it;
+    for (it = data->skippedNames.begin(); 
+	 it != data->skippedNames.end(); it++) {
+	if (fnmatch(it->c_str(), name.c_str(), 0) == 0) {
+	    return true;
+	}
+    }
+    return false;
+}
 
 bool FsTreeWalker::addSkippedPath(const string& ipath)
 {
@@ -108,39 +126,55 @@ bool FsTreeWalker::setSkippedPaths(const list<string> &paths)
     data->skippedPaths.unique();
     return true;
 }
-
-FsTreeWalker::Status FsTreeWalker::walk(const string &top, 
-					FsTreeWalkerCB& cb)
+bool FsTreeWalker::inSkippedPaths(const string& path)
 {
-    return iwalk(path_canon(top), cb);
+    list<string>::const_iterator it;
+    for (it = data->skippedPaths.begin(); 
+	 it != data->skippedPaths.end(); it++) {
+	if (fnmatch(it->c_str(), path.c_str(), FNM_PATHNAME) == 0) 
+	    return true;
+    }
+    return false;
 }
 
-FsTreeWalker::Status FsTreeWalker::iwalk(const string &top, 
+FsTreeWalker::Status FsTreeWalker::walk(const string& _top, 
 					FsTreeWalkerCB& cb)
 {
-    Status status = FtwOk;
+    string top = path_canon(_top);
+
     struct stat st;
     int statret;
-
-    // Handle the top entry
-    statret = (data->options & FtwFollow) ? stat(top.c_str(), &st) :
-	lstat(top.c_str(), &st);
+    // We always follow symlinks at this point. Makes more sense.
+    statret = stat(top.c_str(), &st);
     if (statret == -1) {
 	data->logsyserr("stat", top);
 	return FtwError;
     }
-    if (S_ISDIR(st.st_mode)) {
-	if ((status = cb.processone(top, &st, FtwDirEnter)) & 
+    return iwalk(top, &st, cb);
+}
+
+// Note that the 'norecurse' flag is handled as part of the directory read. 
+// This means that we always go into the top 'walk()' parameter if it is a 
+// directory, even if norecurse is set. Bug or Feature ?
+FsTreeWalker::Status FsTreeWalker::iwalk(const string &top, 
+					 struct stat *stp,
+					 FsTreeWalkerCB& cb)
+{
+    Status status = FtwOk;
+
+    // Handle the parameter
+    if (S_ISDIR(stp->st_mode)) {
+	if ((status = cb.processone(top, stp, FtwDirEnter)) & 
 	    (FtwStop|FtwError)) {
 	    return status;
 	}
-    } else if (S_ISREG(st.st_mode)) {
-	return cb.processone(top, &st, FtwRegular);
+    } else if (S_ISREG(stp->st_mode)) {
+	return cb.processone(top, stp, FtwRegular);
     } else {
 	return status;
     }
 
-    // Handle directory entries
+    // This is a directory, read it and process entries:
     DIR *d = opendir(top.c_str());
     if (d == 0) {
 	data->logsyserr("opendir", top);
@@ -156,20 +190,14 @@ FsTreeWalker::Status FsTreeWalker::iwalk(const string &top,
 
     struct dirent *ent;
     while ((ent = readdir(d)) != 0) {
-	// We do process hidden files for now, only skip . and ..
+	// Skip . and ..
 	if (!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, "..")) 
 	    continue;
 
+	// Skipped file names match ?
 	if (!data->skippedNames.empty()) {
-	    list<string>::const_iterator it;
-	    for (it = data->skippedNames.begin(); 
-		 it != data->skippedNames.end(); it++) {
-		if (fnmatch(it->c_str(), ent->d_name, 0) == 0) {
-		    //fprintf(stderr, 
-		    //"Skipping [%s] because of pattern match\n", ent->d_name);
-		    goto skip;
-		}
-	    }
+	    if (inSkippedNames(ent->d_name))
+		goto skip;
 	}
 
 	{
@@ -183,19 +211,15 @@ FsTreeWalker::Status FsTreeWalker::iwalk(const string &top,
 		continue;
 	    }
 	    if (!data->skippedPaths.empty()) {
-		list<string>::const_iterator it;
-		for (it = data->skippedPaths.begin(); 
-		     it != data->skippedPaths.end(); it++) {
-		    if (fnmatch(it->c_str(), fn.c_str(), FNM_PATHNAME) == 0) 
-			goto skip;
-		}
+		if (inSkippedPaths(fn))
+		    goto skip;
 	    }
 
 	    if (S_ISDIR(st.st_mode)) {
 		if (data->options & FtwNoRecurse) {
 		    status = cb.processone(fn, &st, FtwDirEnter);
 		} else {
-		    status = iwalk(fn, cb);
+		    status = iwalk(fn, &st, cb);
 		}
 		if (status & (FtwStop|FtwError))
 		    goto out;
