@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: readfile.cpp,v 1.3 2006/01/23 13:32:28 dockes Exp $ (C) 2004 J.F.Dockes";
+static char rcsid[] = "@(#$Id: readfile.cpp,v 1.6 2007/09/08 09:44:43 dockes Exp $ (C) 2004 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -32,20 +32,35 @@ using std::string;
 
 #include "readfile.h"
 
-bool file_to_string(const string &fn, string &data)
+static void caterrno(string *reason)
+{
+#define ERRBUFSZ 200    
+    char errbuf[ERRBUFSZ];
+  if (reason) {
+#ifdef sun
+    // Note: sun strerror is noted mt-safe ??
+    *reason += string("file_to_string: open failed: ") + strerror(errno);
+#else
+    strerror_r(errno, errbuf, ERRBUFSZ);
+    *reason += string("file_to_string: open failed: ") + errbuf;
+#endif
+  }
+}
+
+bool file_to_string(const string &fn, string &data, string *reason)
 {
     bool ret = false;
 
     int fd = open(fn.c_str(), O_RDONLY|O_STREAMING);
     if (fd < 0) {
-	// perror("open");
+        caterrno(reason);
 	return false;
     }
     char buf[4096];
     for (;;) {
 	int n = read(fd, buf, 4096);
 	if (n < 0) {
-	    // perror("read");
+	    caterrno(reason);
 	    goto out;
 	}
 	if (n == 0)
@@ -54,7 +69,7 @@ bool file_to_string(const string &fn, string &data)
 	try {
 	    data.append(buf, n);
 	} catch (...) {
-	    //	    fprintf(stderr, "file_to_string: out of memory\n");
+	    caterrno(reason);
 	    goto out;
 	}
     }
